@@ -80,12 +80,6 @@
 //#include "kmplayervdr.h"
 #include "kmplayerconfig.h"
 
-static const int DVDNav_start = 1;
-static const int DVDNav_previous = 2;
-static const int DVDNav_next = 3;
-static const int DVDNav_root = 4;
-static const int DVDNav_up = 5;
-
 extern const char * strMPlayerGroup;
 
 
@@ -118,7 +112,6 @@ KDE_NO_CDTOR_EXPORT KMPlayerApp::KMPlayerApp (QWidget *)
     ListsSource * lstsrc = new ListsSource (m_player);
     m_player->sources () ["listssource"] = lstsrc;
     m_player->sources () ["dvdsource"] = new ::KMPlayerDVDSource(this, m_dvdmenu);
-    m_player->sources () ["dvdnavsource"] = new KMPlayerDVDNavSource (this, m_dvdnavmenu);
     m_player->sources () ["vcdsource"] = new KMPlayerVCDSource(this, m_vcdmenu);
     m_player->sources () ["audiocdsource"] = new KMPlayerAudioCDSource (this, m_audiocdmenu);
     m_player->sources () ["pipesource"] = new KMPlayerPipeSource (this);
@@ -454,12 +447,6 @@ KDE_NO_EXPORT void KMPlayerApp::slotSourceChanged (KMPlayer::Source *olds, KMPla
     }
 }
 
-KDE_NO_EXPORT void KMPlayerApp::dvdNav () {
-    slotStatusMsg(i18n("DVD Navigation..."));
-    m_player->setSource (m_player->sources () ["dvdnavsource"]);
-    slotStatusMsg(i18n("Ready"));
-}
-
 KDE_NO_EXPORT void KMPlayerApp::openDVD () {
     slotStatusMsg(i18n("Opening DVD..."));
     m_player->setSource (m_player->sources () ["dvdsource"]);
@@ -742,7 +729,8 @@ KDE_NO_EXPORT void KMPlayerApp::addUrl (const KUrl& url) {
     KMPlayer::Source * src = m_player->sources () ["urlsource"];
     KMPlayer::NodePtr d = src->document ();
     if (d)
-        d->appendChild (new KMPlayer::GenericURL (d, url.url ()));
+        d->appendChild (new KMPlayer::GenericURL (d,
+                    url.isLocalFile() ? url.toLocalFile() : url.url()));
 }
 
 KDE_NO_EXPORT void KMPlayerApp::saveProperties (KConfigGroup &def_cfg) {
@@ -757,7 +745,7 @@ KDE_NO_EXPORT void KMPlayerApp::readProperties (const KConfigGroup &def_cfg) {
         hide ();
 }
 
-KDE_NO_EXPORT void KMPlayerApp::resizePlayer (int percentage) {
+KDE_NO_EXPORT void KMPlayerApp::resizePlayer (int /*percentage*/) {
     /*KMPlayer::Source * source = m_player->source ();
     if (!source)
         return;
@@ -1083,7 +1071,7 @@ KDE_NO_EXPORT void KMPlayerApp::slotFileOpen () {
         openDocumentFile (urls [0]);
     } else if (urls.size () > 1) {
         m_player->openUrl (KUrl ());
-        for (unsigned int i = 0; i < urls.size (); i++)
+        for (int i = 0; i < urls.size (); i++)
             addUrl (urls [i]);
     }
 }
@@ -1110,7 +1098,7 @@ KDE_NO_EXPORT void KMPlayerApp::slotSaveAs () {
                 QTextStream ts (&file);
                 ts.setEncoding (QTextStream::UnicodeUTF8);
                 ts << QString ("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
-                if (doc->childNodes ()->length () == 1)
+                if (doc->childNodes ().length () == 1)
                     ts << doc->innerXML ();
                 else
                     ts << doc->outerXML ();
@@ -1146,7 +1134,7 @@ KDE_NO_EXPORT void KMPlayerApp::slotGeneratorMenu () {
             KMPlayer::Node *n = gen->firstChild ();
             if (n && n->isElementNode ()) {
                 QString name = static_cast<KMPlayer::Element*>(n)->getAttribute(
-                        KMPlayer::StringPool::attr_name);
+                        KMPlayer::Ids::attr_name);
                 if (name.isEmpty ())
                     name = QFile (files[i]).fileName ();
                 generators.append (new KMPlayer::NodeStoreItem (doc));
@@ -1429,12 +1417,12 @@ KDE_NO_EXPORT void KMPlayerApp::preparePlaylistMenu (KMPlayer::PlayListItem * it
         pm->insertSeparator ();
         manip_node = item->node;
         if (ri->flags & KMPlayer::PlayListView::Deleteable)
-            pm->insertItem (KIconLoader::global ()->loadIconSet (QString ("edit-delete"), KIconLoader::Small, 0, true), i18n ("&Delete item"), this, SLOT (menuDeleteNode ()));
+            pm->insertItem (KIcon ("edit-delete"), i18n ("&Delete item"), this, SLOT (menuDeleteNode ()));
         if (ri->flags & KMPlayer::PlayListView::Moveable) {
             if (manip_node->previousSibling ())
-                pm->insertItem (KIconLoader::global ()->loadIconSet (QString ("go-up"), KIconLoader::Small, 0, true), i18n ("&Move up"), this, SLOT (menuMoveUpNode ()));
+                pm->insertItem (KIcon ("go-up"), i18n ("&Move up"), this, SLOT (menuMoveUpNode ()));
             if (manip_node->nextSibling ())
-                pm->insertItem (KIconLoader::global()->loadIconSet (QString ("go-down"), KIconLoader::Small, 0, true), i18n ("Move &down"), this, SLOT (menuMoveDownNode ()));
+                pm->insertItem (KIcon ("go-down"), i18n ("Move &down"), this, SLOT (menuMoveDownNode ()));
         }
     }
 }
@@ -1553,14 +1541,7 @@ KDE_NO_EXPORT void Disk::activate () {
 KDE_NO_CDTOR_EXPORT KMPlayerDVDSource::KMPlayerDVDSource (KMPlayerApp * a, QMenu * m)
     : KMPlayerMenuSource (i18n ("DVD"), a, m, "dvdsource"), m_configpage (0L) {
     m_menu->insertTearOffHandle ();
-    m_dvdtitlemenu = new QMenu (m_app);
-    m_dvdsubtitlemenu = new QMenu (m_app);
-    m_dvdchaptermenu = new QMenu (m_app);
-    m_dvdlanguagemenu = new QMenu (m_app);
-    m_dvdtitlemenu->setCheckable (true);
-    m_dvdsubtitlemenu->setCheckable (true);
-    m_dvdchaptermenu->setCheckable (true);
-    m_dvdlanguagemenu->setCheckable (true);
+    // FIXME: these menus are void currently
     setUrl ("dvd://");
     m_player->settings ()->addPage (this);
     disks = new Disks (a);
@@ -1579,130 +1560,49 @@ KDE_NO_EXPORT bool KMPlayerDVDSource::processOutput (const QString & str) {
         return true;
     if (m_identified)
         return false;
-    //kDebug () << "scanning " << cstr;
-    QRegExp * patterns = static_cast <KMPlayer::MPlayerPreferencesPage *> (m_player->mediaManager ()->processInfos () ["mplayer"]->config_page)->m_patterns;
-    QRegExp & langRegExp = patterns[KMPlayer::MPlayerPreferencesPage::pat_dvdlang];
-    QRegExp & subtitleRegExp = patterns[KMPlayer::MPlayerPreferencesPage::pat_dvdsub];
-    QRegExp & titleRegExp = patterns[KMPlayer::MPlayerPreferencesPage::pat_dvdtitle];
-    QRegExp & chapterRegExp = patterns[KMPlayer::MPlayerPreferencesPage::pat_dvdchapter];
-    bool post090 = m_player->settings ()->mplayerpost090;
-    if (!post090 && subtitleRegExp.search (str) > -1) {
-        bool ok;
-        int sub_id = subtitleRegExp.cap (1).toInt (&ok);
-        QString sub_title = ok ? subtitleRegExp.cap (2) : subtitleRegExp.cap(1);
-        if (!ok)
-            sub_id = subtitleRegExp.cap (2).toInt (&ok);
-        m_dvdsubtitlemenu->insertItem (sub_title, sub_id);
-        kDebug () << "subtitle sid:" << sub_id << " lang:" << sub_title;
-    } else if (!post090 && langRegExp.search (str) > -1) {
-        bool ok;
-        int lang_id = langRegExp.cap (1).toInt (&ok);
-        QString lang_title = ok ? langRegExp.cap (2) : langRegExp.cap (1);
-        if (!ok)
-            lang_id = langRegExp.cap (2).toInt (&ok);
-        m_dvdlanguagemenu->insertItem (lang_title, lang_id);
-        kDebug () << "lang aid:" << lang_id << " lang:" << lang_title;
-    } else if (titleRegExp.search (str) > -1) {
-        kDebug () << "title " << titleRegExp.cap (1);
-        unsigned ts = titleRegExp.cap (1).toInt ();
-        if ( ts > 100) ts = 100;
-        for (unsigned t = 1; t <= ts; t++)
-            m_dvdtitlemenu->insertItem (QString::number (t), t);
-    } else if (chapterRegExp.search (str) > -1) {
-        kDebug () << "chapter " << chapterRegExp.cap (1);
-        unsigned chs = chapterRegExp.cap (1).toInt ();
-        if ( chs > 100) chs = 100;
-        for (unsigned c = 1; c <= chs; c++)
-            m_dvdchaptermenu->insertItem (QString::number (c), c);
-    } else
-        return false;
-    return true;
+    if (str.startsWith ("ID_DVD_TITLES=")) {
+        int nt = str.mid (14).toInt ();
+        for (int i = 0; i < nt; i++)
+            m_document->appendChild (new KMPlayer::GenericMrl (m_document,
+                        QString ("dvd://%1").arg (i+1),
+                        i18n ("Track %1", QString::number (i+1))));
+        return true;
+    }
+    return false;
 }
 
 KDE_NO_EXPORT void KMPlayerDVDSource::activate () {
     m_start_play = m_auto_play;
-    m_current_title = -1;
     setUrl ("dvd://");
-    m_menu->insertItem (i18n ("&Titles"), m_dvdtitlemenu);
-    m_menu->insertItem (i18n ("&Chapters"), m_dvdchaptermenu);
-    if (!m_player->settings ()->mplayerpost090) {
-        m_menu->insertItem (i18n ("Audio &Language"), m_dvdlanguagemenu);
-        m_menu->insertItem (i18n ("&SubTitles"), m_dvdsubtitlemenu);
-        connect (m_dvdsubtitlemenu, SIGNAL (activated (int)),
-                 this, SLOT (subtitleMenuClicked (int)));
-        connect (m_dvdlanguagemenu, SIGNAL (activated (int)),
-                 this, SLOT (languageMenuClicked (int)));
-    }
-    connect (m_dvdtitlemenu, SIGNAL (activated (int)),
-             this, SLOT (titleMenuClicked (int)));
-    connect (m_dvdchaptermenu, SIGNAL (activated (int)),
-             this, SLOT (chapterMenuClicked (int)));
-    if (m_start_play)
-        QTimer::singleShot (0, m_player, SLOT (play ()));
+    QTimer::singleShot (0, m_player, SLOT (play ()));
 }
 
 KDE_NO_EXPORT void KMPlayerDVDSource::setIdentified (bool b) {
     KMPlayer::Source::setIdentified (b);
     m_start_play = true;
-    if (m_current_title < 0 || m_current_title >= int (m_dvdtitlemenu->count()))
-        m_current_title = 0;
-    if (m_dvdtitlemenu->count ())
-        m_dvdtitlemenu->setItemChecked (m_current_title, true);
-    else
-        m_current_title = -1; // hmmm
-    if (m_dvdchaptermenu->count ()) m_dvdchaptermenu->setItemChecked (0, true);
-    // TODO remember lang/subtitles settings
-    if (m_dvdlanguagemenu->count())
-        m_dvdlanguagemenu->setItemChecked (m_dvdlanguagemenu->idAt (0), true);
+    m_player->updateTree ();
     m_app->slotStatusMsg (i18n ("Ready."));
 }
 
 KDE_NO_EXPORT void KMPlayerDVDSource::deactivate () {
     if (m_player->view ()) {
-        m_dvdtitlemenu->clear ();
-        m_dvdsubtitlemenu->clear ();
-        m_dvdchaptermenu->clear ();
-        m_dvdlanguagemenu->clear ();
         m_menu->removeItemAt (m_menu->count () - 1);
         m_menu->removeItemAt (m_menu->count () - 1);
-        if (!m_player->settings ()->mplayerpost090) {
-            m_menu->removeItemAt (m_menu->count () - 1);
-            m_menu->removeItemAt (m_menu->count () - 1);
-            disconnect (m_dvdsubtitlemenu, SIGNAL (activated (int)),
-                        this, SLOT (subtitleMenuClicked (int)));
-            disconnect (m_dvdlanguagemenu, SIGNAL (activated (int)),
-                        this, SLOT (languageMenuClicked (int)));
-        }
-        disconnect (m_dvdtitlemenu, SIGNAL (activated (int)),
-                    this, SLOT (titleMenuClicked (int)));
-        disconnect (m_dvdchaptermenu, SIGNAL (activated (int)),
-                    this, SLOT (chapterMenuClicked (int)));
     }
 }
 
 KDE_NO_EXPORT void KMPlayerDVDSource::setCurrent (KMPlayer::Mrl *cur) {
     KMPlayer::Source::setCurrent (cur);
     QString url ("dvd://");
-    if (m_document) {
-        if (m_current_title > 0)
-            url += QString::number (m_current_title);
+    if (m_document)
         m_document->mrl ()->src = url;
-    } else
+    else
         setUrl (url);
     m_options = QString (m_identified ? "" : "-v ");
-    if (m_identified) {
-        for (unsigned i = 0; i < m_dvdsubtitlemenu->count (); i++)
-            if (m_dvdsubtitlemenu->isItemChecked (m_dvdsubtitlemenu->idAt (i)))
-                m_options += "-sid " + QString::number (m_dvdsubtitlemenu->idAt(i));
-        for (unsigned i = 0; i < m_dvdchaptermenu->count (); i++)
-            if (m_dvdchaptermenu->isItemChecked (i))
-                m_options += QString (" -chapter %1").arg (i);
-        for (unsigned i = 0; i < m_dvdlanguagemenu->count (); i++)
-            if (m_dvdlanguagemenu->isItemChecked (m_dvdlanguagemenu->idAt (i)))
-                m_options += " -aid " + QString::number(m_dvdlanguagemenu->idAt(i));
-        if (m_player->settings ()->dvddevice.size () > 0)
-            m_options += QString(" -dvd-device ") + m_player->settings()->dvddevice;
-    }
+    if (m_player->settings ()->dvddevice.size () > 0)
+        m_options += QString(" -dvd-device ") + m_player->settings()->dvddevice;
+    if (!m_start_play)
+        m_options += " -frames 0";
     m_recordcmd = m_options + QString (" -vf scale -zoom");
 }
 
@@ -1713,40 +1613,8 @@ KDE_NO_EXPORT QString KMPlayerDVDSource::filterOptions () {
     return QString ("");
 }
 
-KDE_NO_EXPORT void KMPlayerDVDSource::titleMenuClicked (int id) {
-    if (m_current_title != id) {
-        m_player->stop ();
-        m_current_title = id;
-        m_identified = false;
-        m_dvdtitlemenu->clear ();
-        m_dvdsubtitlemenu->clear ();
-        m_dvdchaptermenu->clear ();
-        m_dvdlanguagemenu->clear ();
-        if (m_start_play)
-            QTimer::singleShot (0, m_player, SLOT (play ()));
-    }
-}
-
-KDE_NO_EXPORT void KMPlayerDVDSource::play () {
-    if (m_start_play) {
-        m_player->stop ();
-        QTimer::singleShot (0, m_player, SLOT (play ()));
-    }
-}
-
-KDE_NO_EXPORT void KMPlayerDVDSource::subtitleMenuClicked (int id) {
-    menuItemClicked (m_dvdsubtitlemenu, id);
-    play ();
-}
-
-KDE_NO_EXPORT void KMPlayerDVDSource::languageMenuClicked (int id) {
-    menuItemClicked (m_dvdlanguagemenu, id);
-    play ();
-}
-
-KDE_NO_EXPORT void KMPlayerDVDSource::chapterMenuClicked (int id) {
-    menuItemClicked (m_dvdchaptermenu, id);
-    play ();
+KDE_NO_EXPORT void KMPlayerDVDSource::play (KMPlayer::Mrl *mrl) {
+        KMPlayerMenuSource::play (mrl);
 }
 
 KDE_NO_EXPORT QString KMPlayerDVDSource::prettyName () {
@@ -1786,66 +1654,6 @@ KDE_NO_EXPORT QFrame * KMPlayerDVDSource::prefPage (QWidget * parent) {
 
 //-----------------------------------------------------------------------------
 
-KDE_NO_CDTOR_EXPORT KMPlayerDVDNavSource::KMPlayerDVDNavSource (KMPlayerApp * app, QMenu * m)
-    : KMPlayerMenuSource (i18n ("DVDNav"), app, m, "dvdnavsource") {
-    m_menu->insertTearOffHandle (-1, 0);
-    setUrl ("dvd://");
-}
-
-KDE_NO_CDTOR_EXPORT KMPlayerDVDNavSource::~KMPlayerDVDNavSource () {}
-
-KDE_NO_EXPORT void KMPlayerDVDNavSource::activate () {
-    setUrl ("dvd://");
-    play ();
-}
-
-KDE_NO_EXPORT void KMPlayerDVDNavSource::deactivate () {
-}
-
-KDE_NO_EXPORT void KMPlayerDVDNavSource::play () {
-    if (!m_menu->findItem (DVDNav_previous)) {
-        m_menu->insertItem (i18n ("&Previous"), this, SLOT (navMenuClicked (int)), 0, DVDNav_previous);
-        m_menu->insertItem (i18n ("&Next"), this, SLOT (navMenuClicked (int)), 0, DVDNav_next);
-        m_menu->insertItem (i18n ("&Root"), this, SLOT (navMenuClicked (int)), 0, DVDNav_root);
-        m_menu->insertItem (i18n ("&Up"), this, SLOT (navMenuClicked (int)), 0, DVDNav_up);
-    }
-    QTimer::singleShot (0, m_player, SLOT (play ()));
-    connect (this, SIGNAL (stopPlaying ()), this, SLOT(finished ()));
-}
-
-KDE_NO_EXPORT void KMPlayerDVDNavSource::finished () {
-    disconnect (this, SIGNAL (stopPlaying ()), this, SLOT(finished ()));
-    m_menu->removeItem (DVDNav_previous);
-    m_menu->removeItem (DVDNav_next);
-    m_menu->removeItem (DVDNav_root);
-    m_menu->removeItem (DVDNav_up);
-}
-
-KDE_NO_EXPORT void KMPlayerDVDNavSource::navMenuClicked (int id) {
-    /*switch (id) {
-        case DVDNav_start:
-            break;
-        case DVDNav_previous:
-            m_app->view ()->viewer ()->sendKeyEvent ('p');
-            break;
-        case DVDNav_next:
-            m_app->view ()->viewer ()->sendKeyEvent ('n');
-            break;
-        case DVDNav_root:
-            m_app->view ()->viewer ()->sendKeyEvent ('r');
-            break;
-        case DVDNav_up:
-            m_app->view ()->viewer ()->sendKeyEvent ('u');
-            break;
-    }*/
-}
-
-KDE_NO_EXPORT QString KMPlayerDVDNavSource::prettyName () {
-    return i18n ("DVD");
-}
-
-//-----------------------------------------------------------------------------
-
 KDE_NO_CDTOR_EXPORT KMPlayerPrefSourcePageVCD::KMPlayerPrefSourcePageVCD (QWidget * parent)
  : QFrame (parent) {
      QVBoxLayout *layout = new QVBoxLayout (this, 5, 2);
