@@ -18,31 +18,34 @@
 
 #include <algorithm>
 #include <functional>
-#include <config.h>
+#include "config-kmplayer.h"
 #include <qcheckbox.h>
 #include <qtextedit.h>
 #include <qpushbutton.h>
 #include <qradiobutton.h>
-#include <qmultilineedit.h>
 #include <qtabwidget.h>
 #include <qslider.h>
 #include <qspinbox.h>
 #include <qlabel.h>
-#include <qbuttongroup.h>
 #include <qfileinfo.h>
+#include <Q3ListBox>
+#include <Q3ButtonGroup>
 
 #include <kurlrequester.h>
 #include <klineedit.h>
 #include <kstatusbar.h>
 
 #include <kconfig.h>
+#include <kconfiggroup.h>
 #include <kapplication.h>
 #include <kurl.h>
 #include <kdebug.h>
 #include <klocale.h>
+#include <knuminput.h>
 #include <kcombobox.h>
 #include <kmessagebox.h>
 #include <kglobalsettings.h>
+#include <kcolorscheme.h>
 
 #include "kmplayersource.h"
 #include "kmplayerconfig.h"
@@ -66,7 +69,8 @@ static OutputDriver _ads[] = {
     { "alsa5", i18n ("Advanced Linux Sound Architecture v0.5") },
     { "alsa9", i18n ("Advanced Linux Sound Architecture v0.9") },
     { "", i18n ("Use back-end defaults") },
-    { 0, QString::null }
+    { "pulse", i18n ("PulseAudio") },
+    { 0, QString () }
 };
 
 static OutputDriver _vds [] = {
@@ -78,26 +82,30 @@ static OutputDriver _vds [] = {
     { "gl", i18n ("OpenGL") },
     { "gl2", i18n ("OpenGL MT") },
     { "xv", i18n ("XVideo") },
-    { 0, QString::null }
+    { "vdpau", i18n ("Video Decode and Presentation API for Unix") },
+    { 0, QString () }
 };
 
 static const int ADRIVER_ARTS_INDEX = 4;
 
 
-KDE_NO_CDTOR_EXPORT Settings::Settings (PartBase * player, KConfig * config)
+KDE_NO_CDTOR_EXPORT Settings::Settings (PartBase * player, KSharedConfigPtr config)
   : pagelist (0L), configdialog (0L), m_config (config), m_player (player) {
     audiodrivers = _ads;
     videodrivers = _vds;
     colors [ColorSetting::playlist_background].title = i18n ("Playlist background");
     colors [ColorSetting::playlist_background].option = "PlaylistBackground";
-    colors [ColorSetting::playlist_background].color = KGlobalSettings::baseColor ();
+    colors [ColorSetting::playlist_background].color =
+        KColorScheme(QPalette::Active, KColorScheme::View).background().color();
     colors [ColorSetting::playlist_foreground].title = i18n ("Playlist foreground");
     colors [ColorSetting::playlist_foreground].option = "PlaylistForeground";
-    colors [ColorSetting::playlist_foreground].color = KGlobalSettings::textColor();
+    colors [ColorSetting::playlist_foreground].color =
+        KColorScheme(QPalette::Active, KColorScheme::View).foreground().color();
     colors [ColorSetting::console_background].title =i18n("Console background");
     colors [ColorSetting::playlist_active].title = i18n("Playlist active item");
     colors [ColorSetting::playlist_active].option = "PlaylistActive";
-    colors [ColorSetting::playlist_active].color = KGlobalSettings::linkColor();
+    colors [ColorSetting::playlist_active].color =
+        KColorScheme(QPalette::Active, KColorScheme::Selection).foreground().color();
     colors [ColorSetting::console_background].option = "ConsoleBackground";
     colors [ColorSetting::console_background].color = QColor (0, 0, 0);
     colors [ColorSetting::console_foreground].title = i18n ("Console foreground");
@@ -111,10 +119,12 @@ KDE_NO_CDTOR_EXPORT Settings::Settings (PartBase * player, KConfig * config)
     colors [ColorSetting::area_background].color = QColor (0, 0, 0);
     colors [ColorSetting::infowindow_background].title = i18n ("Info window background");
     colors [ColorSetting::infowindow_background].option ="InfoWindowBackground";
-    colors [ColorSetting::infowindow_background].color = KGlobalSettings::baseColor ();
+    colors [ColorSetting::infowindow_background].color =
+        KColorScheme(QPalette::Active, KColorScheme::View).background().color();
     colors [ColorSetting::infowindow_foreground].title = i18n ("Info window foreground");
     colors [ColorSetting::infowindow_foreground].option ="InfoWindowForeground";
-    colors [ColorSetting::infowindow_foreground].color = KGlobalSettings::textColor();
+    colors [ColorSetting::infowindow_foreground].color =
+        colors [ColorSetting::playlist_foreground].color;
     fonts [FontSetting::playlist].title = i18n ("Playlist");
     fonts [FontSetting::playlist].option = "PlaylistFont";
     fonts [FontSetting::playlist].font = KGlobalSettings::generalFont();
@@ -156,7 +166,6 @@ static const char * strAddConfigButton = "Add Configure Button";
 static const char * strAddPlaylistButton = "Add Playlist Button";
 static const char * strAddRecordButton = "Add Record Button";
 static const char * strAddBroadcastButton = "Add Broadcast Button";
-static const char * strPostMPlayer090 = "Post MPlayer 0.90";
 //static const char * strAutoHideSlider = "Auto Hide Slider";
 static const char * strSeekTime = "Forward/Backward Seek Time";
 static const char * strDVDDevice = "DVD Device";
@@ -164,6 +173,7 @@ static const char * strDVDDevice = "DVD Device";
 //static const char * strShowVCD = "Show VCD Menu";
 static const char * strVCDDevice = "VCD Device";
 const char * strUrlBackend = "URL Backend";
+static const char * strClickToPlay = "Click to Play";
 static const char * strAllowHref = "Allow HREF";
 // postproc thingies
 static const char * strPPGroup = "Post processing options";
@@ -213,35 +223,43 @@ KDE_NO_EXPORT void Settings::applyColorSetting (bool only_changed_ones) {
     for (int i = 0; i < int (ColorSetting::last_target); i++)
         if (!only_changed_ones || colors[i].color != colors[i].newcolor) {
             colors[i].color = colors[i].newcolor;
+            QPalette palette;
             switch (ColorSetting::Target (i)) {
                 case ColorSetting::playlist_background:
-                   view->playList()->setPaletteBackgroundColor(colors[i].color);
+                   palette.setColor (view->playList()->backgroundRole(), colors[i].color);
+                   view->playList()->setPalette (palette);
                    break;
                 case ColorSetting::playlist_foreground:
-                   view->playList()->setPaletteForegroundColor(colors[i].color);
+                   palette.setColor (view->playList()->foregroundRole(), colors[i].color);
+                   view->playList()->setPalette (palette);
                    break;
                 case ColorSetting::playlist_active:
                    view->playList()->setActiveForegroundColor (colors[i].color);
                    break;
                 case ColorSetting::console_background:
-                   view->console()->setPaper (QBrush (colors[i].color));
+                   palette.setColor (view->console()->backgroundRole(), colors[i].color);
+                   view->console()->setPalette (palette);
                    break;
                 case ColorSetting::console_foreground:
-                   view->console()->setColor(colors[i].color);
+                   palette.setColor (view->console()->foregroundRole(), colors[i].color);
+                   view->console()->setPalette (palette);
                    break;
                 case ColorSetting::video_background:
-                   view->viewer ()->setBackgroundColor (colors[i].color);
+                   //palette.setColor (view->viewer()->backgroundRole(), colors[i].color);
+                   //view->viewer()->setPalette (palette);
                    break;
                 case ColorSetting::area_background:
-                   view->viewArea()->setPaletteBackgroundColor(colors[i].color);
+                   palette.setColor (view->viewArea()->backgroundRole(), colors[i].color);
+                   view->viewArea()->setPalette (palette);
                    break;
                 case ColorSetting::infowindow_background:
-                   view->infoPanel ()->setPaper (QBrush (colors[i].color));
-                  break;
+                   palette.setColor(view->infoPanel()->backgroundRole(), colors[i].color);
+                   view->infoPanel()->setPalette (palette);
+                   break;
                 case ColorSetting::infowindow_foreground:
-                  view->infoPanel()->setPaletteForegroundColor(colors[i].color);
-                  view->infoPanel ()->setColor (colors[i].color);
-                  break;
+                   palette.setColor(view->infoPanel()->foregroundRole(), colors[i].color);
+                   view->infoPanel()->setPalette (palette);
+                   break;
                 default:
                     ;
             }
@@ -267,89 +285,89 @@ View * Settings::defaultView () {
 }
 
 KDE_NO_EXPORT void Settings::readConfig () {
-    m_config->setGroup (strGeneralGroup);
-    no_intro = m_config->readBoolEntry (strNoIntro, false);
-    urllist = m_config->readListEntry (strURLList, ';');
-    sub_urllist = m_config->readListEntry (strSubURLList, ';');
-    prefbitrate = m_config->readNumEntry (strPrefBitRate, 512);
-    maxbitrate = m_config->readNumEntry (strMaxBitRate, 1024);
-    volume = m_config->readNumEntry (strVolume, 20);
-    contrast = m_config->readNumEntry (strContrast, 0);
-    brightness = m_config->readNumEntry (strBrightness, 0);
-    hue = m_config->readNumEntry (strHue, 0);
-    saturation = m_config->readNumEntry (strSaturation, 0);
-    const QMap <QString, Source*>::const_iterator e = m_player->sources ().end ();
-    QMap <QString, Source *>::const_iterator i = m_player->sources().begin ();
+    KConfigGroup general (m_config, strGeneralGroup);
+    no_intro = general.readEntry (strNoIntro, false);
+    urllist = general.readEntry (strURLList, QStringList());
+    sub_urllist = general.readEntry (strSubURLList, QStringList());
+    prefbitrate = general.readEntry (strPrefBitRate, 512);
+    maxbitrate = general.readEntry (strMaxBitRate, 1024);
+    volume = general.readEntry (strVolume, 20);
+    contrast = general.readEntry (strContrast, 0);
+    brightness = general.readEntry (strBrightness, 0);
+    hue = general.readEntry (strHue, 0);
+    saturation = general.readEntry (strSaturation, 0);
+    const QMap <QString, Source*>::const_iterator e = m_player->sources ().constEnd ();
+    QMap <QString, Source *>::const_iterator i = m_player->sources().constBegin ();
     for (; i != e; ++i)
-        backends[i.data()->name ()] = m_config->readEntry (i.data()->name ());
+        backends[i.data()->name ()] = general.readEntry (i.data()->name ());
     for (int i = 0; i < int (ColorSetting::last_target); i++)
-        colors[i].newcolor = colors[i].color = m_config->readColorEntry (colors[i].option, &colors[i].color);
+        colors[i].newcolor = colors[i].color = general.readEntry (colors[i].option, colors[i].color);
     for (int i = 0; i < int (FontSetting::last_target); i++)
-        fonts[i].newfont = fonts[i].font = m_config->readFontEntry (fonts[i].option, &fonts[i].font);
+        fonts[i].newfont = fonts[i].font = general.readEntry (fonts[i].option, fonts[i].font);
 
-    m_config->setGroup (strMPlayerGroup);
-    sizeratio = m_config->readBoolEntry (strKeepSizeRatio, true);
-    remembersize = m_config->readBoolEntry (strRememberSize, true);
-    autoresize = m_config->readBoolEntry (strAutoResize, true);
-    docksystray = m_config->readBoolEntry (strDockSysTray, true);
-    loop = m_config->readBoolEntry (strLoop, false);
-    framedrop = m_config->readBoolEntry (strFrameDrop, true);
-    autoadjustvolume = m_config->readBoolEntry (strAdjustVolume, true);
-    autoadjustcolors = m_config->readBoolEntry (strAdjustColors, true);
-    mplayerpost090 = m_config->readBoolEntry (strPostMPlayer090, true);
-    showcnfbutton = m_config->readBoolEntry (strAddConfigButton, true);
-    showrecordbutton = m_config->readBoolEntry (strAddRecordButton, true);
-    showbroadcastbutton = m_config->readBoolEntry (strAddBroadcastButton, true);
-    showplaylistbutton = m_config->readBoolEntry (strAddPlaylistButton, true);
-    seektime = m_config->readNumEntry (strSeekTime, 10);
-    dvddevice = m_config->readEntry (strDVDDevice, "/dev/dvd");
-    vcddevice = m_config->readEntry (strVCDDevice, "/dev/cdrom");
-    videodriver = m_config->readNumEntry (strVoDriver, 0);
-    audiodriver = m_config->readNumEntry (strAoDriver, 0);
-    allowhref = m_config->readBoolEntry(strAllowHref, false);
+    KConfigGroup mplayer (m_config, strMPlayerGroup);
+    sizeratio = mplayer.readEntry (strKeepSizeRatio, true);
+    remembersize = mplayer.readEntry (strRememberSize, true);
+    autoresize = mplayer.readEntry (strAutoResize, true);
+    docksystray = mplayer.readEntry (strDockSysTray, true);
+    loop = mplayer.readEntry (strLoop, false);
+    framedrop = mplayer.readEntry (strFrameDrop, true);
+    autoadjustvolume = mplayer.readEntry (strAdjustVolume, true);
+    autoadjustcolors = mplayer.readEntry (strAdjustColors, true);
+    showcnfbutton = mplayer.readEntry (strAddConfigButton, true);
+    showrecordbutton = mplayer.readEntry (strAddRecordButton, true);
+    showbroadcastbutton = mplayer.readEntry (strAddBroadcastButton, true);
+    showplaylistbutton = mplayer.readEntry (strAddPlaylistButton, true);
+    seektime = mplayer.readEntry (strSeekTime, 10);
+    dvddevice = mplayer.readEntry (strDVDDevice, "/dev/dvd");
+    vcddevice = mplayer.readEntry (strVCDDevice, "/dev/cdrom");
+    videodriver = mplayer.readEntry (strVoDriver, 0);
+    audiodriver = mplayer.readEntry (strAoDriver, 0);
+    clicktoplay = mplayer.readEntry (strClickToPlay, false);
+    grabhref = mplayer.readEntry (strAllowHref, false);
 
     // recording
-    m_config->setGroup (strRecordingGroup);
-    mencoderarguments = m_config->readEntry (strMencoderArgs, "-oac mp3lame -ovc lavc");
-    ffmpegarguments = m_config->readEntry (strFFMpegArgs, "-f avi -acodec mp3 -vcodec mpeg4");
-    recordfile = m_config->readPathEntry(strRecordingFile, QDir::homeDirPath () + "/record.avi");
-    recorder = Recorder (m_config->readNumEntry (strRecorder, int (MEncoder)));
-    replayoption = ReplayOption (m_config->readNumEntry (strAutoPlayAfterRecording, ReplayFinished));
-    replaytime = m_config->readNumEntry (strAutoPlayAfterTime, 60);
-    recordcopy = m_config->readBoolEntry(strRecordingCopy, true);
+    KConfigGroup rec_cfg (m_config, strRecordingGroup);
+    mencoderarguments = rec_cfg.readEntry (strMencoderArgs, QString ("-oac mp3lame -ovc lavc"));
+    ffmpegarguments = rec_cfg.readEntry (strFFMpegArgs, QString ("-f avi -acodec mp3 -vcodec mpeg4"));
+    recordfile = rec_cfg.readPathEntry(strRecordingFile, QDir::homeDirPath () + "/record.avi");
+    recorder = Recorder (rec_cfg.readEntry (strRecorder, int (MEncoder)));
+    replayoption = ReplayOption (rec_cfg.readEntry (strAutoPlayAfterRecording, int (ReplayFinished)));
+    replaytime = rec_cfg.readEntry (strAutoPlayAfterTime, 60);
+    recordcopy = rec_cfg.readEntry(strRecordingCopy, true);
 
     // postproc
-    m_config->setGroup (strPPGroup);
-    postprocessing = m_config->readBoolEntry (strPostProcessing, false);
-    disableppauto = m_config->readBoolEntry (strDisablePPauto, true);
+    KConfigGroup pp_cfg (m_config, strPPGroup);
+    postprocessing = pp_cfg.readEntry (strPostProcessing, false);
+    disableppauto = pp_cfg.readEntry (strDisablePPauto, true);
 
-    pp_default = m_config->readBoolEntry (strPP_Default, true);
-    pp_fast = m_config->readBoolEntry (strPP_Fast, false);
-    pp_custom = m_config->readBoolEntry (strPP_Custom, false);
+    pp_default = pp_cfg.readEntry (strPP_Default, true);
+    pp_fast = pp_cfg.readEntry (strPP_Fast, false);
+    pp_custom = pp_cfg.readEntry (strPP_Custom, false);
     // default these to default preset
-    pp_custom_hz = m_config->readBoolEntry (strCustom_Hz, true);
-    pp_custom_hz_aq = m_config->readBoolEntry (strCustom_Hz_Aq, true);
-    pp_custom_hz_ch = m_config->readBoolEntry (strCustom_Hz_Ch, false);
+    pp_custom_hz = pp_cfg.readEntry (strCustom_Hz, true);
+    pp_custom_hz_aq = pp_cfg.readEntry (strCustom_Hz_Aq, true);
+    pp_custom_hz_ch = pp_cfg.readEntry (strCustom_Hz_Ch, false);
 
-    pp_custom_vt = m_config->readBoolEntry (strCustom_Vt, true);
-    pp_custom_vt_aq = m_config->readBoolEntry (strCustom_Vt_Aq, true);
-    pp_custom_vt_ch = m_config->readBoolEntry (strCustom_Vt_Ch, false);
+    pp_custom_vt = pp_cfg.readEntry (strCustom_Vt, true);
+    pp_custom_vt_aq = pp_cfg.readEntry (strCustom_Vt_Aq, true);
+    pp_custom_vt_ch = pp_cfg.readEntry (strCustom_Vt_Ch, false);
 
-    pp_custom_dr = m_config->readBoolEntry (strCustom_Dr, true);
-    pp_custom_dr_aq = m_config->readBoolEntry (strCustom_Dr_Aq, true);
-    pp_custom_dr_ch = m_config->readBoolEntry (strCustom_Dr_Ch, false);
+    pp_custom_dr = pp_cfg.readEntry (strCustom_Dr, true);
+    pp_custom_dr_aq = pp_cfg.readEntry (strCustom_Dr_Aq, true);
+    pp_custom_dr_ch = pp_cfg.readEntry (strCustom_Dr_Ch, false);
 
-    pp_custom_al = m_config->readBoolEntry (strCustom_Al, true);
-    pp_custom_al_f = m_config->readBoolEntry (strCustom_Al_F, false);
+    pp_custom_al = pp_cfg.readEntry (strCustom_Al, true);
+    pp_custom_al_f = pp_cfg.readEntry (strCustom_Al_F, false);
 
-    pp_custom_tn = m_config->readBoolEntry (strCustom_Tn, true);
-    pp_custom_tn_s = m_config->readNumEntry (strCustom_Tn_S, 0);
+    pp_custom_tn = pp_cfg.readEntry (strCustom_Tn, true);
+    pp_custom_tn_s = pp_cfg.readEntry (strCustom_Tn_S, 0);
 
-    pp_lin_blend_int = m_config->readBoolEntry (strPP_Lin_Blend_Int, false);
-    pp_lin_int = m_config->readBoolEntry (strPP_Lin_Int, false);
-    pp_cub_int = m_config->readBoolEntry (strPP_Cub_Int, false);
-    pp_med_int = m_config->readBoolEntry (strPP_Med_Int, false);
-    pp_ffmpeg_int = m_config->readBoolEntry (strPP_FFmpeg_Int, false);
+    pp_lin_blend_int = pp_cfg.readEntry (strPP_Lin_Blend_Int, false);
+    pp_lin_int = pp_cfg.readEntry (strPP_Lin_Int, false);
+    pp_cub_int = pp_cfg.readEntry (strPP_Cub_Int, false);
+    pp_med_int = pp_cfg.readEntry (strPP_Med_Int, false);
+    pp_ffmpeg_int = pp_cfg.readEntry (strPP_FFmpeg_Int, false);
 
     for (PreferencesPage * p = pagelist; p; p = p->next)
         p->read (m_config);
@@ -360,11 +378,13 @@ KDE_NO_EXPORT bool Settings::createDialog () {
     if (configdialog) return false;
     configdialog = new Preferences (m_player, this);
     int id = 0;
-    const PartBase::ProcessMap::const_iterator e = m_player->players ().end ();
-    for (PartBase::ProcessMap::const_iterator i = m_player->players ().begin(); i != e; ++i) {
-        Process * p = i.data ();
-        if (p->supports ("urlsource"))
-            configdialog->m_SourcePageURL->backend->insertItem (p->menuName ().remove (QChar ('&')), id++);
+    const MediaManager::ProcessInfoMap::const_iterator e = m_player->mediaManager()->processInfos ().constEnd ();
+    for (MediaManager::ProcessInfoMap::const_iterator i = m_player->mediaManager()->processInfos ().constBegin(); i != e; ++i) {
+        ProcessInfo *p = i.data ();
+        if (p->supports ("urlsource")) {
+            QString lbl = p->label.remove (QChar ('&'));
+            configdialog->m_SourcePageURL->backend->insertItem (lbl, id++);
+        }
     }
     connect (configdialog, SIGNAL (okClicked ()),
             this, SLOT (okPressed ()));
@@ -402,7 +422,7 @@ void Settings::removePage (PreferencesPage * page) {
             break;
         }
 }
-    
+
 void Settings::show (const char * pagename) {
     bool created = createDialog ();
     configdialog->m_GeneralPageGeneral->keepSizeRatio->setChecked (sizeratio);
@@ -425,10 +445,10 @@ void Settings::show (const char * pagename) {
         fonts[i].newfont = fonts[i].font;
     configdialog->m_SourcePageURL->urllist->clear ();
     configdialog->m_SourcePageURL->urllist->insertStringList (urllist);
-    configdialog->m_SourcePageURL->urllist->setCurrentText (m_player->source ()->url ().prettyURL ());
+    configdialog->m_SourcePageURL->urllist->setCurrentText (m_player->source ()->url ().prettyUrl ());
     configdialog->m_SourcePageURL->sub_urllist->clear ();
     configdialog->m_SourcePageURL->sub_urllist->insertStringList (sub_urllist);
-    configdialog->m_SourcePageURL->sub_urllist->setCurrentText (m_player->source ()->subUrl ().prettyURL ());
+    configdialog->m_SourcePageURL->sub_urllist->setCurrentText (m_player->source ()->subUrl ().prettyUrl ());
     configdialog->m_SourcePageURL->changed = false;
     configdialog->m_SourcePageURL->prefBitRate->setText (QString::number (prefbitrate));
     configdialog->m_SourcePageURL->maxBitRate->setText (QString::number (maxbitrate));
@@ -437,16 +457,17 @@ void Settings::show (const char * pagename) {
     configdialog->m_GeneralPageOutput->audioDriver->setCurrentItem (audiodriver);
     configdialog->m_SourcePageURL->backend->setCurrentItem (configdialog->m_SourcePageURL->backend->findItem (backends["urlsource"]));
     int id = 0;
-    const PartBase::ProcessMap::const_iterator e = m_player->players ().end ();
-    for (PartBase::ProcessMap::const_iterator i = m_player->players ().begin(); i != e; ++i) {
-        Process * p = i.data ();
+    const MediaManager::ProcessInfoMap::const_iterator e = m_player->mediaManager()->processInfos ().constEnd ();
+    for (MediaManager::ProcessInfoMap::const_iterator i = m_player->mediaManager()->processInfos ().constBegin(); i != e; ++i) {
+        ProcessInfo *p = i.data ();
         if (p->supports ("urlsource")) {
-            if (backends["urlsource"] == QString (p->name()))
+            if (backends["urlsource"] == QString (p->name))
                 configdialog->m_SourcePageURL->backend->setCurrentItem (id);
             id++;
         }
     }
-    configdialog->m_SourcePageURL->allowhref->setChecked (allowhref);
+    configdialog->m_SourcePageURL->clicktoplay->setChecked (clicktoplay);
+    configdialog->m_SourcePageURL->grabhref->setChecked (grabhref);
 
     // postproc
     configdialog->m_OPPagePostproc->postProcessing->setChecked (postprocessing);
@@ -485,7 +506,7 @@ void Settings::show (const char * pagename) {
     configdialog->m_RecordPage->recorder->setButton (int (recorder));
     configdialog->m_RecordPage->replayClicked (int (replayoption));
     configdialog->m_RecordPage->recorderClicked (int (recorder));
-    configdialog->m_RecordPage->replaytime->setText (QString::number (replaytime));
+    configdialog->m_RecordPage->replaytime->setValue (replaytime);
     configdialog->m_MEncoderPage->arguments->setText (mencoderarguments);
     configdialog->m_MEncoderPage->format->setButton (recordcopy ? 0 : 1);
     configdialog->m_MEncoderPage->formatClicked (recordcopy ? 0 : 1);
@@ -503,85 +524,86 @@ void Settings::show (const char * pagename) {
 }
 
 void Settings::writeConfig () {
-    m_config->setGroup (strGeneralGroup);
-    m_config->writeEntry (strURLList, urllist, ';');
-    m_config->writeEntry (strSubURLList, sub_urllist, ';');
-    m_config->writeEntry (strPrefBitRate, prefbitrate);
-    m_config->writeEntry (strMaxBitRate, maxbitrate);
-    m_config->writeEntry (strVolume, volume);
-    m_config->writeEntry (strContrast, contrast);
-    m_config->writeEntry (strBrightness, brightness);
-    m_config->writeEntry (strHue, hue);
-    m_config->writeEntry (strSaturation, saturation);
-    const QMap<QString,QString>::iterator b_end = backends.end ();
-    for (QMap<QString,QString>::iterator i = backends.begin(); i != b_end; ++i)
-        m_config->writeEntry (i.key (), i.data ());
+    KConfigGroup gen_cfg (m_config, strGeneralGroup);
+    gen_cfg.writeEntry (strURLList, urllist);
+    gen_cfg.writeEntry (strSubURLList, sub_urllist);
+    gen_cfg.writeEntry (strPrefBitRate, prefbitrate);
+    gen_cfg.writeEntry (strMaxBitRate, maxbitrate);
+    gen_cfg.writeEntry (strVolume, volume);
+    gen_cfg.writeEntry (strContrast, contrast);
+    gen_cfg.writeEntry (strBrightness, brightness);
+    gen_cfg.writeEntry (strHue, hue);
+    gen_cfg.writeEntry (strSaturation, saturation);
+    const QMap<QString,QString>::ConstIterator b_end = backends.constEnd ();
+    for (QMap<QString,QString>::ConstIterator i = backends.constBegin(); i != b_end; ++i)
+        gen_cfg.writeEntry (i.key (), i.data ());
     for (int i = 0; i < int (ColorSetting::last_target); i++)
-        m_config->writeEntry (colors[i].option, colors[i].color);
+        gen_cfg.writeEntry (colors[i].option, colors[i].color);
     for (int i = 0; i < int (FontSetting::last_target); i++)
-        m_config->writeEntry (fonts[i].option, fonts[i].font);
-    m_config->setGroup (strMPlayerGroup);
-    m_config->writeEntry (strKeepSizeRatio, sizeratio);
-    m_config->writeEntry (strAutoResize, autoresize);
-    m_config->writeEntry (strRememberSize, remembersize);
-    m_config->writeEntry (strDockSysTray, docksystray);
-    m_config->writeEntry (strLoop, loop);
-    m_config->writeEntry (strFrameDrop, framedrop);
-    m_config->writeEntry (strAdjustVolume, autoadjustvolume);
-    m_config->writeEntry (strAdjustColors, autoadjustcolors);
-    m_config->writeEntry (strSeekTime, seektime);
-    m_config->writeEntry (strVoDriver, videodriver);
-    m_config->writeEntry (strAoDriver, audiodriver);
-    m_config->writeEntry (strAllowHref, allowhref);
-    m_config->writeEntry (strAddConfigButton, showcnfbutton);
-    m_config->writeEntry (strAddPlaylistButton, showplaylistbutton);
-    m_config->writeEntry (strAddRecordButton, showrecordbutton);
-    m_config->writeEntry (strAddBroadcastButton, showbroadcastbutton);
+        gen_cfg.writeEntry (fonts[i].option, fonts[i].font);
 
-    m_config->writeEntry (strDVDDevice, dvddevice);
-    m_config->writeEntry (strVCDDevice, vcddevice);
+    KConfigGroup mplayer_cfg (m_config, strMPlayerGroup);
+    mplayer_cfg.writeEntry (strKeepSizeRatio, sizeratio);
+    mplayer_cfg.writeEntry (strAutoResize, autoresize);
+    mplayer_cfg.writeEntry (strRememberSize, remembersize);
+    mplayer_cfg.writeEntry (strDockSysTray, docksystray);
+    mplayer_cfg.writeEntry (strLoop, loop);
+    mplayer_cfg.writeEntry (strFrameDrop, framedrop);
+    mplayer_cfg.writeEntry (strAdjustVolume, autoadjustvolume);
+    mplayer_cfg.writeEntry (strAdjustColors, autoadjustcolors);
+    mplayer_cfg.writeEntry (strSeekTime, seektime);
+    mplayer_cfg.writeEntry (strVoDriver, videodriver);
+    mplayer_cfg.writeEntry (strAoDriver, audiodriver);
+    mplayer_cfg.writeEntry (strClickToPlay, clicktoplay);
+    mplayer_cfg.writeEntry (strAllowHref, grabhref);
+    mplayer_cfg.writeEntry (strAddConfigButton, showcnfbutton);
+    mplayer_cfg.writeEntry (strAddPlaylistButton, showplaylistbutton);
+    mplayer_cfg.writeEntry (strAddRecordButton, showrecordbutton);
+    mplayer_cfg.writeEntry (strAddBroadcastButton, showbroadcastbutton);
+    mplayer_cfg.writeEntry (strDVDDevice, dvddevice);
+    mplayer_cfg.writeEntry (strVCDDevice, vcddevice);
 
     //postprocessing stuff
-    m_config->setGroup (strPPGroup);
-    m_config->writeEntry (strPostProcessing, postprocessing);
-    m_config->writeEntry (strDisablePPauto, disableppauto);
-    m_config->writeEntry (strPP_Default, pp_default);
-    m_config->writeEntry (strPP_Fast, pp_fast);
-    m_config->writeEntry (strPP_Custom, pp_custom);
+    KConfigGroup pp_cfg (m_config, strPPGroup);
+    pp_cfg.writeEntry (strPostProcessing, postprocessing);
+    pp_cfg.writeEntry (strDisablePPauto, disableppauto);
+    pp_cfg.writeEntry (strPP_Default, pp_default);
+    pp_cfg.writeEntry (strPP_Fast, pp_fast);
+    pp_cfg.writeEntry (strPP_Custom, pp_custom);
 
-    m_config->writeEntry (strCustom_Hz, pp_custom_hz);
-    m_config->writeEntry (strCustom_Hz_Aq, pp_custom_hz_aq);
-    m_config->writeEntry (strCustom_Hz_Ch, pp_custom_hz_ch);
+    pp_cfg.writeEntry (strCustom_Hz, pp_custom_hz);
+    pp_cfg.writeEntry (strCustom_Hz_Aq, pp_custom_hz_aq);
+    pp_cfg.writeEntry (strCustom_Hz_Ch, pp_custom_hz_ch);
 
-    m_config->writeEntry (strCustom_Vt, pp_custom_vt);
-    m_config->writeEntry (strCustom_Vt_Aq, pp_custom_vt_aq);
-    m_config->writeEntry (strCustom_Vt_Ch, pp_custom_vt_ch);
+    pp_cfg.writeEntry (strCustom_Vt, pp_custom_vt);
+    pp_cfg.writeEntry (strCustom_Vt_Aq, pp_custom_vt_aq);
+    pp_cfg.writeEntry (strCustom_Vt_Ch, pp_custom_vt_ch);
 
-    m_config->writeEntry (strCustom_Dr, pp_custom_dr);
-    m_config->writeEntry (strCustom_Dr_Aq, pp_custom_vt_aq);
-    m_config->writeEntry (strCustom_Dr_Ch, pp_custom_vt_ch);
+    pp_cfg.writeEntry (strCustom_Dr, pp_custom_dr);
+    pp_cfg.writeEntry (strCustom_Dr_Aq, pp_custom_vt_aq);
+    pp_cfg.writeEntry (strCustom_Dr_Ch, pp_custom_vt_ch);
 
-    m_config->writeEntry (strCustom_Al, pp_custom_al);
-    m_config->writeEntry (strCustom_Al_F, pp_custom_al_f);
+    pp_cfg.writeEntry (strCustom_Al, pp_custom_al);
+    pp_cfg.writeEntry (strCustom_Al_F, pp_custom_al_f);
 
-    m_config->writeEntry (strCustom_Tn, pp_custom_tn);
-    m_config->writeEntry (strCustom_Tn_S, pp_custom_tn_s);
+    pp_cfg.writeEntry (strCustom_Tn, pp_custom_tn);
+    pp_cfg.writeEntry (strCustom_Tn_S, pp_custom_tn_s);
 
-    m_config->writeEntry (strPP_Lin_Blend_Int, pp_lin_blend_int);
-    m_config->writeEntry (strPP_Lin_Int, pp_lin_int);
-    m_config->writeEntry (strPP_Cub_Int, pp_cub_int);
-    m_config->writeEntry (strPP_Med_Int, pp_med_int);
-    m_config->writeEntry (strPP_FFmpeg_Int, pp_ffmpeg_int);
+    pp_cfg.writeEntry (strPP_Lin_Blend_Int, pp_lin_blend_int);
+    pp_cfg.writeEntry (strPP_Lin_Int, pp_lin_int);
+    pp_cfg.writeEntry (strPP_Cub_Int, pp_cub_int);
+    pp_cfg.writeEntry (strPP_Med_Int, pp_med_int);
+    pp_cfg.writeEntry (strPP_FFmpeg_Int, pp_ffmpeg_int);
 
     // recording
-    m_config->setGroup (strRecordingGroup);
-    m_config->writePathEntry (strRecordingFile, recordfile);
-    m_config->writeEntry (strAutoPlayAfterRecording, int (replayoption));
-    m_config->writeEntry (strAutoPlayAfterTime, replaytime);
-    m_config->writeEntry (strRecorder, int (recorder));
-    m_config->writeEntry (strRecordingCopy, recordcopy);
-    m_config->writeEntry (strMencoderArgs, mencoderarguments);
-    m_config->writeEntry (strFFMpegArgs, ffmpegarguments);
+    KConfigGroup rec_cfg (m_config, strRecordingGroup);
+    rec_cfg.writePathEntry (strRecordingFile, recordfile);
+    rec_cfg.writeEntry (strAutoPlayAfterRecording, int (replayoption));
+    rec_cfg.writeEntry (strAutoPlayAfterTime, replaytime);
+    rec_cfg.writeEntry (strRecorder, int (recorder));
+    rec_cfg.writeEntry (strRecordingCopy, recordcopy);
+    rec_cfg.writeEntry (strMencoderArgs, mencoderarguments);
+    rec_cfg.writeEntry (strFFMpegArgs, ffmpegarguments);
 
     //dynamic stuff
     for (PreferencesPage * p = pagelist; p; p = p->next)
@@ -590,50 +612,52 @@ void Settings::writeConfig () {
     m_config->sync ();
 }
 
-KDE_NO_EXPORT void Settings::okPressed () {
+void Settings::okPressed () {
     bool urlchanged = configdialog->m_SourcePageURL->changed;
     bool playerchanged = false;
+    KUrl url = configdialog->m_SourcePageURL->url->url ();
+    KUrl sub_url = configdialog->m_SourcePageURL->sub_url->url ();
     if (urlchanged) {
-        if (configdialog->m_SourcePageURL->url->url ().isEmpty ())
+        if (url.isEmpty ()) {
             urlchanged = false;
-        else {
-            if (KURL::fromPathOrURL (configdialog->m_SourcePageURL->url->url ()).isLocalFile () ||
-                    KURL::isRelativeURL (configdialog->m_SourcePageURL->url->url ())) {
-                QFileInfo fi (configdialog->m_SourcePageURL->url->url ());
-                int hpos = configdialog->m_SourcePageURL->url->url ().findRev ('#');
+        } else {
+            if (KUrl (url.url ()).isLocalFile () || KUrl::isRelativeUrl (url.url ())) {
+                QFileInfo fi (url.path ());
+                int hpos = url.url ().lastIndexOf ('#');
                 QString xine_directives ("");
                 while (!fi.exists () && hpos > -1) {
-                    xine_directives = configdialog->m_SourcePageURL->url->url ().mid (hpos);
-                    fi.setFile (configdialog->m_SourcePageURL->url->url ().left (hpos));
-                    hpos = configdialog->m_SourcePageURL->url->url ().findRev ('#', hpos-1);
+                    xine_directives = url.url ().mid (hpos);
+                    fi.setFile (url.url ().left (hpos));
+                    hpos = url.url ().lastIndexOf ('#', hpos-1);
                 }
                 if (!fi.exists ()) {
                     urlchanged = false;
-                    KMessageBox::error (m_player->view (), i18n ("File %1 does not exist.").arg (configdialog->m_SourcePageURL->url->url ()), i18n ("Error"));
-                } else
-                    configdialog->m_SourcePageURL->url->setURL (fi.absFilePath () + xine_directives);
+                    KMessageBox::error (m_player->view (), i18n ("File %1 does not exist.",url.url ()), i18n ("Error"));
+                } else {
+                    configdialog->m_SourcePageURL->url->setUrl (fi.absFilePath () + xine_directives);
+                }
             }
             if (urlchanged &&
-                    !configdialog->m_SourcePageURL->sub_url->url ().isEmpty () &&
-                    (KURL::fromPathOrURL (configdialog->m_SourcePageURL->sub_url->url ()).isLocalFile () ||
-                     KURL::isRelativeURL (configdialog->m_SourcePageURL->sub_url->url ()))) {
-                QFileInfo sfi (configdialog->m_SourcePageURL->sub_url->url ());
+                    !sub_url.url ().isEmpty () &&
+                    (KUrl (sub_url.url ()).isLocalFile () ||
+                     KUrl::isRelativeUrl (sub_url.url ()))) {
+                QFileInfo sfi (sub_url.path ());
                 if (!sfi.exists ()) {
-                    KMessageBox::error (m_player->view (), i18n ("Sub title file %1 does not exist.").arg (configdialog->m_SourcePageURL->sub_url->url ()), i18n ("Error"));
-                    configdialog->m_SourcePageURL->sub_url->setURL (QString::null);
+                    KMessageBox::error (m_player->view (), i18n ("Sub title file %1 does not exist.",sub_url.url ()), i18n ("Error"));
+                    configdialog->m_SourcePageURL->sub_url->setUrl (QString ());
                 } else
-                    configdialog->m_SourcePageURL->sub_url->setURL (sfi.absFilePath ());
+                    configdialog->m_SourcePageURL->sub_url->setUrl (sfi.absFilePath ());
             }
         }
     }
     if (urlchanged) {
-        KURL url = KURL::fromPathOrURL (configdialog->m_SourcePageURL->url->url ());
-        m_player->setURL (url);
-        if (urllist.find (url.prettyURL ()) == urllist.end ())
-            configdialog->m_SourcePageURL->urllist->insertItem (url.prettyURL (), 0);
-        KURL sub_url = KURL::fromPathOrURL (configdialog->m_SourcePageURL->sub_url->url ());
-        if (sub_urllist.find (sub_url.prettyURL ()) == sub_urllist.end ())
-            configdialog->m_SourcePageURL->sub_urllist->insertItem (sub_url.prettyURL (), 0);
+        KUrl uri (url.url ());
+        m_player->setUrl (uri.url ());
+        if (urllist.find (uri.prettyUrl ()) == urllist.end ())
+            configdialog->m_SourcePageURL->urllist->insertItem (uri.prettyUrl (), 0);
+        KUrl sub_uri (sub_url.url ());
+        if (sub_urllist.find (sub_uri.prettyUrl ()) == sub_urllist.end ())
+            configdialog->m_SourcePageURL->sub_urllist->insertItem (sub_uri.prettyUrl (), 0);
     }
     urllist.clear ();
     for (int i = 0; i < configdialog->m_SourcePageURL->urllist->count () && i < 20; ++i)
@@ -663,21 +687,20 @@ KDE_NO_EXPORT void Settings::okPressed () {
 
     videodriver = configdialog->m_GeneralPageOutput->videoDriver->currentItem();
     audiodriver = configdialog->m_GeneralPageOutput->audioDriver->currentItem();
-    if (!strcmp (m_player->source()->name (), "urlsource")) {
-        int backend = configdialog->m_SourcePageURL->backend->currentItem ();
-        const PartBase::ProcessMap::const_iterator e = m_player->players ().end();
-        for (PartBase::ProcessMap::const_iterator i = m_player->players ().begin(); backend >=0 && i != e; ++i) {
-            Process * proc = i.data ();
-            if (proc->supports ("urlsource") && backend-- == 0) {
-                backends["urlsource"] = proc->name ();
-                if (proc != m_player->process ()) {
-                    m_player->setProcess (proc->name ());
-                    playerchanged = true;
-                }
+    QString backend_name = configdialog->m_SourcePageURL->backend->currentText ();
+    if (!backend_name.isEmpty ()) {
+        const MediaManager::ProcessInfoMap::const_iterator e = m_player->mediaManager()->processInfos ().constEnd ();
+        for (MediaManager::ProcessInfoMap::const_iterator i = m_player->mediaManager()->processInfos ().constBegin(); i != e; ++i) {
+            ProcessInfo *p = i.data ();
+            if (p->supports ("urlsource") &&
+                    p->label.remove (QChar ('&')) == backend_name) {
+                backends["urlsource"] = p->name;
+                break;
             }
         }
     }
-    allowhref = configdialog->m_SourcePageURL->allowhref->isChecked ();
+    clicktoplay = configdialog->m_SourcePageURL->clicktoplay->isChecked ();
+    grabhref = configdialog->m_SourcePageURL->grabhref->isChecked ();
     //postproc
     postprocessing = configdialog->m_OPPagePostproc->postProcessing->isChecked();
     disableppauto = configdialog->m_OPPagePostproc->disablePPauto->isChecked();
@@ -716,7 +739,7 @@ KDE_NO_EXPORT void Settings::okPressed () {
     recorder = Recorder (configdialog->m_RecordPage->recorder->selectedId ());
 #endif
     replaytime = configdialog->m_RecordPage->replaytime->text ().toInt ();
-    configdialog->m_RecordPage->replaytime->setText (QString::number (replaytime));
+    configdialog->m_RecordPage->replaytime->setValue (replaytime);
     recordfile = configdialog->m_RecordPage->url->lineEdit()->text ();
     mencoderarguments = configdialog->m_MEncoderPage->arguments->text ();
     ffmpegarguments = configdialog->m_FFMpegPage->arguments->text ();
@@ -736,14 +759,14 @@ KDE_NO_EXPORT void Settings::okPressed () {
 
     if (urlchanged || playerchanged) {
         m_player->sources () ["urlsource"]->setSubURL
-            (KURL(configdialog->m_SourcePageURL->sub_url->url()));
-        m_player->openURL (KURL::fromPathOrURL (configdialog->m_SourcePageURL->url->url ()));
-        m_player->source ()->setSubURL (KURL::fromPathOrURL (configdialog->m_SourcePageURL->sub_url->url ()));
+            (KUrl(configdialog->m_SourcePageURL->sub_url->url()));
+        m_player->openUrl (KUrl (configdialog->m_SourcePageURL->url->url ()));
+        m_player->source ()->setSubURL (KUrl (configdialog->m_SourcePageURL->sub_url->url ()));
     }
 }
 
 KDE_NO_EXPORT void Settings::getHelp () {
-    KApplication::kApplication()->invokeBrowser ("man:/mplayer");
+   // KApplication::kApplication()->invokeBrowser ("man:/mplayer");
 }
 
 #include "kmplayerconfig.moc"
