@@ -26,17 +26,13 @@
 #include <kurl.h>
 
 #include "kmplayerplaylist.h"
-#include "kmplayerprocess.h"
 
-class KConfig;
-class QWidget;
-class QFrame;
-class QListViewItem;
 
 namespace KMPlayer {
-    
+
 class PartBase;
 
+
 /**
  * Class for a certain media, like URL, DVD, TV etc
  */
@@ -60,11 +56,10 @@ public:
     /* position () returns position in deci-seconds */
     KDE_NO_EXPORT int position () const { return m_position; }
     KDE_NO_EXPORT float aspect () const { return m_aspect; }
-    KDE_NO_EXPORT const KURL & url () const { return m_url; }
-    KDE_NO_EXPORT const KURL & subUrl () const { return m_sub_url; }
+    KDE_NO_EXPORT const KUrl & url () const { return m_url; }
+    KDE_NO_EXPORT const KUrl & subUrl () const { return m_sub_url; }
     PartBase * player () { return m_player; }
     virtual void reset ();
-    QString currentMrl ();
     KDE_NO_EXPORT const QString & audioDevice () const { return m_audiodevice; }
     KDE_NO_EXPORT const QString & videoDevice () const { return m_videodevice; }
     KDE_NO_EXPORT const QString & videoNorm () const { return m_videonorm; }
@@ -76,14 +71,18 @@ public:
     KDE_NO_EXPORT const QString & options () const { return m_options; }
     KDE_NO_EXPORT const QString & recordCmd () const { return m_recordcmd; }
     KDE_NO_EXPORT const QString & tuner () const { return m_tuner; }
-    KDE_NO_EXPORT NodePtr current () const { return m_current; }
+    KDE_NO_EXPORT Mrl *current() { return m_current ? m_current->mrl() : NULL;}
+    virtual void setCurrent (Mrl *mrl);
+    QString plugin (const QString &mime) const;
     virtual NodePtr document ();
+    void setDocument (KMPlayer::NodePtr doc, KMPlayer::NodePtr cur);
     virtual NodePtr root ();
     virtual QString filterOptions ();
+    virtual bool authoriseUrl (const QString &url);
 
-    void setURL (const KURL & url);
+    virtual void setUrl (const QString &url);
     void insertURL (NodePtr mrl, const QString & url, const QString & title=QString());
-    KDE_NO_EXPORT void setSubURL (const KURL & url) { m_sub_url = url; }
+    KDE_NO_EXPORT void setSubURL (const KUrl & url) { m_sub_url = url; }
     void setLanguages (const QStringList & alang, const QStringList & slang);
     KDE_NO_EXPORT void setWidth (int w) { m_width = w; }
     KDE_NO_EXPORT void setHeight (int h) { m_height = h; }
@@ -94,20 +93,15 @@ public:
     /* setPosition (pos) set position in deci-seconds */
     void setPosition (int pos);
     virtual void setIdentified (bool b = true);
-    // backend process state changed
-    virtual void stateChange (Process *, Process::State os, Process::State ns);
     KDE_NO_EXPORT void setAutoPlay (bool b) { m_auto_play = b; }
     KDE_NO_EXPORT bool autoPlay () const { return m_auto_play; }
     void setTitle (const QString & title);
     void setLoading (int percentage);
-    bool setCurrent (NodePtr mrl);
 
     virtual QString prettyName ();
 signals:
     void startPlaying ();
     void stopPlaying ();
-    void startRecording ();
-    void stopRecording ();
     /**
      * Signal for notifying this source is at the end of play items
      */
@@ -119,14 +113,10 @@ public slots:
     virtual void deactivate () = 0;
     virtual void forward ();
     virtual void backward ();
-    virtual void play ();
     /**
-     * Continuing playing where current is now
-     * May call play process if a video needs to play or 
-     * emit endOfPlayItems when done
+     * Play at node position
      */
-    virtual void playCurrent ();
-    virtual void jump (NodePtr e);
+    virtual void play (Mrl *);
     void setAudioLang (int);
     void setSubtitle (int);
 protected:
@@ -134,27 +124,21 @@ protected:
     /**
      * PlayListNotify implementation
      */
-    bool requestPlayURL (NodePtr mrl);
-    bool resolveURL (NodePtr mrl);
     void stateElementChanged (Node * element, Node::State os, Node::State ns);
-    void setEventDispatcher (NodePtr element);
-    void setInfoMessage (const QString & msg);
-    void repaintRect (int x, int y, int w, int h);
-    void moveRect (int x, int y, int w, int h, int x1, int y1);
-    void avWidgetSizes (int x, int y, int w, int h, unsigned int * bg_color);
     void bitRates (int & preferred, int & maximal);
     void setTimeout (int ms);
+    void openUrl (const KUrl &url, const QString &target, const QString &srv);
+    void enableRepaintUpdaters (bool enable, unsigned int off_time);
 
     NodePtr m_document;
     NodePtrW m_current;
-    NodePtrW m_back_request;
     QString m_name;
     PartBase * m_player;
     QString m_recordcmd;
     bool m_identified;
     bool m_auto_play;
-    KURL m_url;
-    KURL m_sub_url;
+    KUrl m_url;
+    KUrl m_sub_url;
     QString m_audiodevice;
     QString m_videodevice;
     QString m_videonorm;
@@ -164,6 +148,7 @@ protected:
     int m_xvencoding;
     QString m_pipecmd;
     QString m_options;
+    QString m_plugin;
 private:
     int m_width;
     int m_height;
@@ -173,6 +158,17 @@ private:
     int m_doc_timer;
 };
 
+class KMPLAYER_EXPORT SourceDocument : public Document {
+public:
+    SourceDocument (Source *s, const QString &url);
+
+    void message (MessageType msg, void *data=NULL);
+    void *role (RoleType msg, void *data=NULL);
+protected:
+    Source *m_source;
+    ConnectionList m_KeyListeners;
+};
+
 } // namespace
 
 #endif
