@@ -22,16 +22,15 @@
 #include <qtimer.h>
 #include <qpushbutton.h>
 #include <qcheckbox.h>
-#include <qtable.h>
+#include <Q3Table>
 #include <qstringlist.h>
 #include <qcombobox.h>
 #include <qlineedit.h>
 #include <qgroupbox.h>
 #include <qwhatsthis.h>
 #include <qtabwidget.h>
-#include <qbuttongroup.h>
 #include <qmessagebox.h>
-#include <qpopupmenu.h>
+#include <QMenu>
 #include <qfontmetrics.h>
 
 #include <klocale.h>
@@ -41,6 +40,7 @@
 #include <kurlrequester.h>
 #include <kcombobox.h>
 #include <kconfig.h>
+#include <kconfiggroup.h>
 #include <kstandarddirs.h>
 
 #include "kmplayerpartbase.h"
@@ -48,9 +48,9 @@
 #include "kmplayerconfig.h"
 #include "kmplayertvsource.h"
 #include "playlistview.h"
+#include "viewarea.h"
 #include "kmplayer.h"
 #include "kmplayercontrolpanel.h"
-#include "kmplayerbroadcast.h"
 
 static const char * strTV = "TV";
 static const char * strTVDriver = "Driver";
@@ -64,18 +64,18 @@ KDE_NO_CDTOR_EXPORT TVDevicePage::TVDevicePage (QWidget *parent, KMPlayer::NodeP
     layout->addWidget (deviceLabel);
     QGridLayout *gridlayout = new QGridLayout (layout, 3, 4);
     QLabel * audioLabel = new QLabel (i18n ("Audio device:"), this);
-    audiodevice = new KURLRequester (device->getAttribute ("audio"), this);
+    audiodevice = new KUrlRequester (KUrl (device->getAttribute ("audio")), this);
     QLabel * nameLabel = new QLabel (i18n ("Name:"), this, 0);
-    name = new QLineEdit (device->pretty_name, this, 0);
+    name = new QLineEdit (device->title, this, 0);
     QLabel *sizewidthLabel = new QLabel (i18n ("Width:"), this, 0);
-    sizewidth = new QLineEdit (device->getAttribute (KMPlayer::StringPool::attr_width), this, 0);
+    sizewidth = new QLineEdit (device->getAttribute (KMPlayer::Ids::attr_width), this, 0);
     QLabel *sizeheightLabel = new QLabel (i18n ("Height:"), this, 0);
-    sizeheight = new QLineEdit (device->getAttribute (KMPlayer::StringPool::attr_height), this, 0);
+    sizeheight = new QLineEdit (device->getAttribute (KMPlayer::Ids::attr_height), this, 0);
     noplayback = new QCheckBox (i18n ("Do not immediately play"), this);
     noplayback->setChecked (!device->getAttribute ("playback").toInt ());
     QWhatsThis::add (noplayback, i18n ("Only start playing after clicking the play button"));
     inputsTab = new QTabWidget (this);
-    for (KMPlayer::NodePtr ip = device->firstChild (); ip; ip = ip->nextSibling ()) {
+    for (KMPlayer::Node *ip = device->firstChild (); ip; ip = ip->nextSibling ()) {
         if (ip->id != id_node_tv_input)
             continue;
         TVInput * input = KMPlayer::convertNode <TVInput> (ip);
@@ -93,28 +93,28 @@ KDE_NO_CDTOR_EXPORT TVDevicePage::TVDevicePage (QWidget *parent, KMPlayer::NodeP
             horzlayout->addWidget (norms);
             vertlayout->addLayout (horzlayout);
             vertlayout->addItem (new QSpacerItem (0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding));
-            QTable * table = new QTable (90, 2, widget, "PageTVChannels");
+            Q3Table * table = new Q3Table (90, 2, widget, "PageTVChannels");
             QFontMetrics metrics (table->font ());
-            QHeader *header = table->horizontalHeader();
+            Q3Header *header = table->horizontalHeader();
             header->setLabel (0, i18n ("Channel"));
             header->setLabel (1, i18n ("Frequency (MHz)"));
             int index = 0;
             int first_column_width = QFontMetrics (header->font ()).boundingRect (header->label (0)).width () + 20;
-            for (KMPlayer::NodePtr c=input->firstChild();c;c=c->nextSibling()) {
+            for (KMPlayer::Node *c=input->firstChild();c;c=c->nextSibling()) {
                 if (c->id != id_node_tv_channel)
                     continue;
-                int strwid = metrics.boundingRect (c->mrl ()->pretty_name).width ();
+                int strwid = metrics.boundingRect (c->mrl ()->title).width ();
                 if (strwid > first_column_width)
                     first_column_width = strwid + 4;
-                table->setItem (index, 0, new QTableItem (table, QTableItem::Always, c->mrl ()->pretty_name));
-                table->setItem (index++, 1, new QTableItem (table, QTableItem::Always, KMPlayer::convertNode<TVChannel>(c)->getAttribute ("frequency")));
+                table->setItem (index, 0, new Q3TableItem (table, Q3TableItem::Always, c->mrl ()->title));
+                table->setItem (index++, 1, new Q3TableItem (table, Q3TableItem::Always, KMPlayer::convertNode<TVChannel>(c)->getAttribute ("frequency")));
             }
             table->setColumnWidth (0, first_column_width);
             table->setColumnStretchable (1, true);
             tablayout->addWidget (table);
             tablayout->addLayout (vertlayout);
         }
-        inputsTab->addTab (widget, input->mrl ()->pretty_name);
+        inputsTab->addTab (widget, input->mrl ()->title);
     }
     QPushButton * delButton = new QPushButton (i18n ("Delete"), this);
     connect (delButton, SIGNAL (clicked ()), this, SLOT (slotDelete ()));
@@ -156,7 +156,7 @@ KDE_NO_CDTOR_EXPORT KMPlayerPrefSourcePageTV::KMPlayerPrefSourcePageTV (QWidget
     driver = new QLineEdit ("", general, 0);
     QWhatsThis::add (driver, i18n ("dummy, v4l or bsdbt848"));
     QLabel *deviceLabel = new QLabel (i18n ("Device:"), general, 0);
-    device = new KURLRequester ("/dev/video", general);
+    device = new KUrlRequester (KUrl ("/dev/video"), general);
     QWhatsThis::add(device, i18n("Path to your video device, eg. /dev/video0"));
     scan = new QPushButton (i18n ("Scan..."), general);
     gridlayout->addWidget (driverLabel, 0, 0);
@@ -179,18 +179,17 @@ KDE_NO_EXPORT void KMPlayerPrefSourcePageTV::showEvent (QShowEvent *) {
 
 KDE_NO_CDTOR_EXPORT TVNode::TVNode (KMPlayer::NodePtr &d, const QString & s, const char * t, short id, const QString & n) : KMPlayer::GenericMrl (d, s, n, t) {
     this->id = id;
-    editable = true;
 }
 
 KDE_NO_EXPORT void TVNode::setNodeName (const QString & nn) {
-    pretty_name = nn;
-    setAttribute (KMPlayer::StringPool::attr_name, nn);
+    title = nn;
+    setAttribute (KMPlayer::Ids::attr_name, nn);
 }
 
 //-----------------------------------------------------------------------------
 
 KDE_NO_CDTOR_EXPORT TVChannel::TVChannel (KMPlayer::NodePtr & d, const QString & n, double freq) : TVNode (d, QString ("tv://"), "channel", id_node_tv_channel, n) {
-    setAttribute (KMPlayer::StringPool::attr_name, n);
+    setAttribute (KMPlayer::Ids::attr_name, n);
     setAttribute ("frequency", QString::number (freq, 'f', 2));
 }
 
@@ -198,22 +197,23 @@ KDE_NO_CDTOR_EXPORT TVChannel::TVChannel (KMPlayer::NodePtr & d) : TVNode (d, QS
 }
 
 KDE_NO_EXPORT void TVChannel::closed () {
-    pretty_name = getAttribute (KMPlayer::StringPool::attr_name);
+    title = getAttribute (KMPlayer::Ids::attr_name);
+    Mrl::closed ();
 }
 
 //-----------------------------------------------------------------------------
 
 TVInput::TVInput (KMPlayer::NodePtr & d, const QString & n, int id)
  : TVNode (d, QString ("tv://"), "input", id_node_tv_input, n) {
-    setAttribute (KMPlayer::StringPool::attr_name, n);
-    setAttribute (KMPlayer::StringPool::attr_id, QString::number (id));
+    setAttribute (KMPlayer::Ids::attr_name, n);
+    setAttribute (KMPlayer::Ids::attr_id, QString::number (id));
 }
 
 KDE_NO_CDTOR_EXPORT TVInput::TVInput (KMPlayer::NodePtr & d) : TVNode (d, QString ("tv://"), "input", id_node_tv_input) {
 }
 
-KDE_NO_EXPORT KMPlayer::NodePtr TVInput::childFromTag (const QString & tag) {
-    // kdDebug () << nodeName () << " childFromTag " << tag << endl;
+KDE_NO_EXPORT KMPlayer::Node *TVInput::childFromTag (const QString & tag) {
+    // kDebug () << nodeName () << " childFromTag " << tag;
     if (tag == QString::fromLatin1 ("channel")) {
         return new TVChannel (m_doc);
     } else
@@ -221,18 +221,19 @@ KDE_NO_EXPORT KMPlayer::NodePtr TVInput::childFromTag (const QString & tag) {
 }
 
 KDE_NO_EXPORT void TVInput::closed () {
-    //pretty_name = getAttribute (KMPlayer::StringPool::attr_name);
+    //title = getAttribute (KMPlayer::Ids::attr_name);
+    Mrl::closed ();
 }
 
 KDE_NO_EXPORT void TVInput::setNodeName (const QString & name) {
-    Node * p = parentNode ().ptr ();
+    Node *p = parentNode ();
     QString nm (name);
     if (p && p->id == id_node_tv_device) {
-        int pos = name.find (QString (" - ") + p->mrl ()->pretty_name);
+        int pos = name.indexOf (QString (" - ") + p->mrl ()->title);
         if (pos > -1)
             nm.truncate (pos);
     }
-    pretty_name = nm + QString (" - ") + pretty_name;
+    title = nm + QString (" - ") + title;
     TVNode::setNodeName (nm);
 }
 
@@ -251,8 +252,8 @@ KDE_NO_CDTOR_EXPORT TVDevice::~TVDevice () {
         device_page->deleteLater ();
 }
 
-KDE_NO_EXPORT KMPlayer::NodePtr TVDevice::childFromTag (const QString & tag) {
-    // kdDebug () << nodeName () << " childFromTag " << tag << endl;
+KDE_NO_EXPORT KMPlayer::Node *TVDevice::childFromTag (const QString & tag) {
+    // kDebug () << nodeName () << " childFromTag " << tag;
     if (tag == QString::fromLatin1 ("input"))
         return new TVInput (m_doc);
     return 0L;
@@ -260,10 +261,21 @@ KDE_NO_EXPORT KMPlayer::NodePtr TVDevice::childFromTag (const QString & tag) {
 
 KDE_NO_EXPORT void TVDevice::closed () {
     updateNodeName ();
+    Mrl::closed ();
 }
 
-KDE_NO_EXPORT void TVDevice::childDone (KMPlayer::NodePtr) {
-    finish ();
+KDE_NO_EXPORT void TVDevice::message (KMPlayer::MessageType msg, void *data) {
+    if (KMPlayer::MsgChildFinished == msg)
+        finish ();
+    else
+        TVNode::message (msg, data);
+}
+
+void *TVDevice::role (KMPlayer::RoleType msg, void *content)
+{
+    if (KMPlayer::RolePlaylist == msg)
+        return NULL;
+    return TVNode::role (msg, content);
 }
 
 KDE_NO_EXPORT void TVDevice::setNodeName (const QString & name) {
@@ -272,34 +284,34 @@ KDE_NO_EXPORT void TVDevice::setNodeName (const QString & name) {
 }
 
 KDE_NO_EXPORT void TVDevice::updateNodeName () {
-    pretty_name = getAttribute (KMPlayer::StringPool::attr_name);
+    title = getAttribute (KMPlayer::Ids::attr_name);
     src = getAttribute ("path");
-    for (KMPlayer::NodePtr c = firstChild (); c; c = c->nextSibling ())
+    for (KMPlayer::Node *c = firstChild (); c; c = c->nextSibling ())
         if (c->id == id_node_tv_input) {
-            TVInput * i = static_cast <TVInput *> (c.ptr ());
-            i->pretty_name = i->getAttribute (KMPlayer::StringPool::attr_name) +
-                QString (" - ") + pretty_name;
+            TVInput * i = static_cast <TVInput *> (c);
+            i->title = i->getAttribute (KMPlayer::Ids::attr_name) +
+                QString (" - ") + title;
         }
 }
 
 KDE_NO_EXPORT void TVDevice::updateDevicePage () {
     if (!device_page)
         return;
-    pretty_name = device_page->name->text ();
-    setAttribute (KMPlayer::StringPool::attr_name, pretty_name);
+    title = device_page->name->text ();
+    setAttribute (KMPlayer::Ids::attr_name, title);
     setAttribute ("audio", device_page->audiodevice->lineEdit()->text ());
     setAttribute ("playback", device_page->noplayback->isChecked() ? "0" : "1");
-    setAttribute (KMPlayer::StringPool::attr_width, device_page->sizewidth->text ());
-    setAttribute (KMPlayer::StringPool::attr_height, device_page->sizeheight->text ());
+    setAttribute (KMPlayer::Ids::attr_width, device_page->sizewidth->text ());
+    setAttribute (KMPlayer::Ids::attr_height, device_page->sizeheight->text ());
     int i = 0;
-    for (KMPlayer::NodePtr ip = firstChild(); ip; ip=ip->nextSibling(),++i) {
+    for (KMPlayer::Node *ip = firstChild(); ip; ip=ip->nextSibling(),++i) {
         if (ip->id != id_node_tv_input)
             continue;
         TVInput * input = KMPlayer::convertNode <TVInput> (ip);
         bool ok;
         if (input->getAttribute ("tuner").toInt (&ok) && ok) {
             QWidget * widget = device_page->inputsTab->page (i);
-            QTable * table = static_cast <QTable *> (widget->child ("PageTVChannels", "QTable"));
+            Q3Table * table = static_cast <Q3Table *> (widget->child ("PageTVChannels", "Q3Table"));
             if (table) {
                 input->clearChildren ();
                 for (int j = 0; j<table->numRows() && table->item (j, 1); ++j) {
@@ -319,37 +331,40 @@ KDE_NO_EXPORT void TVDevice::updateDevicePage () {
 KDE_NO_CDTOR_EXPORT
 TVDocument::TVDocument (KMPlayerTVSource * source)
     : FileDocument (id_node_tv_document, "tv://", source), m_source (source) {
-    pretty_name = i18n ("Television");
+    title = i18n ("Television");
 }
 
-KDE_NO_EXPORT KMPlayer::NodePtr TVDocument::childFromTag (const QString & tag) {
-    // kdDebug () << nodeName () << " childFromTag " << tag << endl;
+KDE_NO_EXPORT KMPlayer::Node *TVDocument::childFromTag (const QString & tag) {
+    // kDebug () << nodeName () << " childFromTag " << tag;
     if (tag == QString::fromLatin1 ("device"))
         return new TVDevice (m_doc);
     return FileDocument::childFromTag (tag);
 }
 
-KDE_NO_EXPORT void TVDocument::childDone (KMPlayer::NodePtr) {
-    finish ();
+KDE_NO_EXPORT void TVDocument::message (KMPlayer::MessageType msg, void *data) {
+    if (KMPlayer::MsgChildFinished == msg)
+        finish ();
+    else
+        FileDocument::message (msg, data);
 }
 
 KDE_NO_EXPORT void TVDocument::defer () {
     if (!resolved) {
         resolved = true;
-        readFromFile (locateLocal ("data", "kmplayer/tv.xml"));
+        readFromFile (KStandardDirs::locateLocal ("data", "kmplayer/tv.xml"));
     }
 }
 
 //-----------------------------------------------------------------------------
 
-KDE_NO_CDTOR_EXPORT KMPlayerTVSource::KMPlayerTVSource (KMPlayerApp * a, QPopupMenu * m)
+KDE_NO_CDTOR_EXPORT KMPlayerTVSource::KMPlayerTVSource (KMPlayerApp * a, QMenu * m)
     : KMPlayerMenuSource (i18n ("TV"), a, m, "tvsource"), m_configpage (0L), scanner (0L), config_read (false) {
     m_url = "tv://";
     m_menu->insertTearOffHandle ();
     connect (m_menu, SIGNAL (aboutToShow ()), this, SLOT (menuAboutToShow ()));
     m_document = new TVDocument (this);
     m_player->settings ()->addPage (this);
-    tree_id = static_cast <KMPlayer::View*>(m_player->view ())->playList ()->addTree (m_document, "tvsource", "tv", KMPlayer::PlayListView::TreeEdit | KMPlayer::PlayListView::Moveable | KMPlayer::PlayListView::Deleteable);
+    tree_id = static_cast <KMPlayer::View*>(m_player->view ())->playList ()->addTree (m_document, "tvsource", "video-television", KMPlayer::PlayListView::TreeEdit | KMPlayer::PlayListView::Moveable | KMPlayer::PlayListView::Deleteable);
 }
 
 KDE_NO_CDTOR_EXPORT KMPlayerTVSource::~KMPlayerTVSource () {
@@ -357,18 +372,18 @@ KDE_NO_CDTOR_EXPORT KMPlayerTVSource::~KMPlayerTVSource () {
 
 KDE_NO_EXPORT void KMPlayerTVSource::activate () {
     m_identified = true;
-    if (m_player->settings ()->showbroadcastbutton)
-        m_app->view()->controlPanel()->broadcastButton ()->show ();
+    //if (m_player->settings ()->showbroadcastbutton)
+    //    m_app->view()->controlPanel()->broadcastButton ()->show ();
     if (m_cur_tvdevice && !m_current) {
-        for (KMPlayer::NodePtr i = m_cur_tvdevice->firstChild(); i && !m_current; i=i->nextSibling())
+        for (KMPlayer::Node *i = m_cur_tvdevice->firstChild(); i && !m_current; i=i->nextSibling())
             if (i->id == id_node_tv_input) {
                 TVInput * input = KMPlayer::convertNode <TVInput> (i);
                 bool ok;
                 m_cur_tvinput = i;
                 if (input->getAttribute ("tuner").toInt (&ok) && ok) {
-                    for (KMPlayer::NodePtr c = i->firstChild (); c; c = c->nextSibling ())
+                    for (KMPlayer::Node *c = i->firstChild (); c; c = c->nextSibling ())
                         if (c->id == id_node_tv_channel) {
-                            m_current = c;
+                            setCurrent (c->mrl ());
                             break;
                         }
                 } else
@@ -376,7 +391,6 @@ KDE_NO_EXPORT void KMPlayerTVSource::activate () {
             }
     } else if (!m_cur_tvdevice)
         KMPlayer::Source::reset ();
-    buildArguments ();
     if (m_cur_tvdevice) {
         QString playback = static_cast <KMPlayer::Element *> (m_cur_tvdevice.ptr ())->getAttribute (QString::fromLatin1 ("playback"));
         if (playback.isEmpty () || playback.toInt ())
@@ -388,29 +402,29 @@ KDE_NO_EXPORT void KMPlayerTVSource::activate () {
  */
 
 KDE_NO_EXPORT void KMPlayerTVSource::deactivate () {
-    if (m_player->view () && !m_app->view ()->controlPanel()->broadcastButton ()->isOn ())
-        m_app->view ()->controlPanel()->broadcastButton ()->hide ();
+    //if (m_player->view () && !m_app->view ()->controlPanel()->broadcastButton ()->isOn ())
+    //    m_app->view ()->controlPanel()->broadcastButton ()->hide ();
     reset ();
 }
 
 KDE_NO_EXPORT void KMPlayerTVSource::buildMenu () {
     m_menu->clear ();
     int counter = 0;
-    for (KMPlayer::NodePtr dp = m_document->firstChild (); dp; dp = dp->nextSibling ())
+    for (KMPlayer::Node *dp = m_document->firstChild (); dp; dp = dp->nextSibling ())
         if (dp->id == id_node_tv_device)
-            m_menu->insertItem (KMPlayer::convertNode <TVDevice> (dp)->pretty_name, this, SLOT (menuClicked (int)), 0, counter++);
+            m_menu->insertItem (KMPlayer::convertNode <TVDevice> (dp)->title, this, SLOT (menuClicked (int)), 0, counter++);
 }
 
 KDE_NO_EXPORT void KMPlayerTVSource::menuAboutToShow () {
     readXML ();
 }
 
-void KMPlayerTVSource::jump (KMPlayer::NodePtr e) {
-    if (e->id == id_node_tv_document) {
+void KMPlayerTVSource::play (KMPlayer::Mrl *mrl) {
+    if (mrl && mrl->id == id_node_tv_document) {
         readXML ();
     } else {
-        m_current = e;
-        for (; e; e = e->parentNode ()) {
+        m_current = mrl;
+        for (KMPlayer::Node *e = mrl; e; e = e->parentNode ()) {
             if (e->id == id_node_tv_device) {
                 m_cur_tvdevice = e;
                 break;
@@ -419,11 +433,18 @@ void KMPlayerTVSource::jump (KMPlayer::NodePtr e) {
         }
         if (m_player->source () != this)
             m_player->setSource (this);
-        else if (m_player->process ()->playing ()) {
-            m_back_request = m_current;
+        else
+            KMPlayer::Source::play (mrl);
+        /*else if (m_player->process ()->playing ()) {
+            //m_back_request = m_current;
             m_player->process ()->stop ();
-        } else
-            playCurrent ();
+        } else {
+            buildArguments ();
+            if (m_app->broadcasting ())
+                QTimer::singleShot (0, m_app->broadcastConfig (), SLOT (startFeed ()));
+            else
+                KMPlayer::Source::play (mrl);
+        }*/
     }
 }
 
@@ -431,9 +452,10 @@ KDE_NO_EXPORT KMPlayer::NodePtr KMPlayerTVSource::root () {
     return m_cur_tvinput;
 }
 
-KDE_NO_EXPORT void KMPlayerTVSource::buildArguments () {
+KDE_NO_EXPORT void KMPlayerTVSource::setCurrent (KMPlayer::Mrl *mrl) {
     TVChannel * channel = 0L;
     TVInput * input = 0L;
+    m_current = mrl;
     KMPlayer::NodePtr elm = m_current;
     if (elm && elm->id == id_node_tv_channel) {
         channel = KMPlayer::convertNode <TVChannel> (elm);
@@ -454,7 +476,7 @@ KDE_NO_EXPORT void KMPlayerTVSource::buildArguments () {
     m_audiodevice = tvdevice->getAttribute ("audio");
     m_videodevice = tvdevice->src;
     m_videonorm = input->getAttribute ("norm");
-    m_tuner = input->getAttribute (KMPlayer::StringPool::attr_name);
+    m_tuner = input->getAttribute (KMPlayer::Ids::attr_name);
     QString xvport = tvdevice->getAttribute ("xvport");
     if (!xvport.isEmpty ())
         m_xvport = xvport.toInt ();
@@ -463,8 +485,8 @@ KDE_NO_EXPORT void KMPlayerTVSource::buildArguments () {
         m_xvencoding = xvenc.toInt ();
     QString command;
     command.sprintf ("device=%s:input=%s",
-            tvdevice->src.ascii (),
-            input->getAttribute (KMPlayer::StringPool::attr_id).ascii ());
+            tvdevice->src.toAscii ().data (),
+            input->getAttribute (KMPlayer::Ids::attr_id).toAscii ().data ());
     if (channel) {
         QString freq = channel->getAttribute ("frequency");
         m_frequency = (int)(1000 * freq.toDouble ());
@@ -473,27 +495,16 @@ KDE_NO_EXPORT void KMPlayerTVSource::buildArguments () {
         m_frequency = 0;
     if (!m_videonorm.isEmpty ())
         command += QString (":norm=%1").arg (m_videonorm);
-    m_app->setCaption (i18n ("TV: ") + (channel ? channel->mrl ()->pretty_name : input->mrl ()->pretty_name), false);
+    m_app->setCaption (i18n ("TV: ") + (channel ? channel->mrl ()->title : input->mrl ()->title), false);
     setDimensions (m_cur_tvdevice,
-            tvdevice->getAttribute (KMPlayer::StringPool::attr_width).toInt (),
-            tvdevice->getAttribute (KMPlayer::StringPool::attr_height).toInt ());
-    m_options.sprintf ("-tv noaudio:driver=%s:%s:width=%d:height=%d -slave -nocache -quiet", tvdriver.ascii (), command.ascii (), width (), height ());
-    if (m_player->settings ()->mplayerpost090)
-        m_recordcmd.sprintf ("-tv %s:driver=%s:%s:width=%d:height=%d", m_audiodevice.isEmpty () ? "noaudio" : (QString ("forceaudio:adevice=") + m_audiodevice).ascii(), tvdriver.ascii (), command.ascii (), width (), height ());
-    else
-        m_recordcmd.sprintf ("-tv on:%s:driver=%s:%s:width=%d:height=%d", m_audiodevice.isEmpty () ? "noaudio" : (QString ("forceaudio:adevice=") + m_audiodevice).ascii(), tvdriver.ascii (), command.ascii (), width (), height ());
-}
-
-KDE_NO_EXPORT void KMPlayerTVSource::playCurrent () {
-    buildArguments ();
-    if (m_app->broadcasting ())
-        QTimer::singleShot (0, m_app->broadcastConfig (), SLOT (startFeed ()));
-    else
-        KMPlayer::Source::playCurrent ();
+            tvdevice->getAttribute (KMPlayer::Ids::attr_width).toInt (),
+            tvdevice->getAttribute (KMPlayer::Ids::attr_height).toInt ());
+    m_options.sprintf ("-tv noaudio:driver=%s:%s:width=%d:height=%d -slave -nocache -quiet", tvdriver.toAscii ().data (), command.toAscii ().data (), width (), height ());
+    m_recordcmd.sprintf ("-tv %s:driver=%s:%s:width=%d:height=%d", m_audiodevice.isEmpty () ? "noaudio" : (QString ("forceaudio:adevice=") + m_audiodevice).toAscii ().data(), tvdriver.toAscii ().data (), command.toAscii ().data (), width (), height ());
 }
 
 KDE_NO_EXPORT void KMPlayerTVSource::menuClicked (int id) {
-    KMPlayer::NodePtr elm = m_document->firstChild ();
+    KMPlayer::Node *elm = m_document->firstChild ();
     for (; id > 0; --id,  elm = elm->nextSibling ())
         ;
     m_cur_tvdevice = elm;
@@ -505,7 +516,7 @@ KDE_NO_EXPORT void KMPlayerTVSource::menuClicked (int id) {
 KDE_NO_EXPORT QString KMPlayerTVSource::filterOptions () {
     if (! m_player->settings ()->disableppauto)
         return KMPlayer::Source::filterOptions ();
-    return QString ("-vop pp=lb");
+    return QString ("-vf pp=lb");
 }
 
 KDE_NO_EXPORT bool KMPlayerTVSource::hasLength () {
@@ -523,28 +534,27 @@ KDE_NO_EXPORT QString KMPlayerTVSource::prettyName () {
     return name;
 }
 
-KDE_NO_EXPORT void KMPlayerTVSource::write (KConfig * m_config) {
+KDE_NO_EXPORT void KMPlayerTVSource::write (KSharedConfigPtr m_config) {
     if (!config_read) return;
-    m_config->setGroup (strTV);
-    m_config->writeEntry (strTVDriver, tvdriver);
+    KConfigGroup (m_config, strTV).writeEntry (strTVDriver, tvdriver);
     static_cast <TVDocument *> (m_document.ptr ())->writeToFile
-        (locateLocal ("data", "kmplayer/tv.xml"));
-    kdDebug () << "KMPlayerTVSource::write XML" << endl;
+        (KStandardDirs::locateLocal ("data", "kmplayer/tv.xml"));
+    kDebug () << "KMPlayerTVSource::write XML";
 }
 
 KDE_NO_EXPORT void KMPlayerTVSource::readXML () {
     if (config_read) return;
     config_read = true;
-    kdDebug () << "KMPlayerTVSource::readXML" << endl;
+    kDebug () << "KMPlayerTVSource::readXML";
     m_document->defer ();
     static_cast <KMPlayer::View*>(m_player->view ())->playList ()->updateTree (tree_id, m_document, 0, false, false);
     buildMenu ();
     sync (false);
 }
 
-KDE_NO_EXPORT void KMPlayerTVSource::read (KConfig * m_config) {
-    m_config->setGroup (strTV);
-    tvdriver = m_config->readEntry (strTVDriver, "v4l");
+KDE_NO_EXPORT void KMPlayerTVSource::read (KSharedConfigPtr m_config) {
+    tvdriver = KConfigGroup (m_config, strTV).readEntry (
+            strTVDriver, QString ("v4l2"));
 }
 
 KDE_NO_EXPORT void KMPlayerTVSource::sync (bool fromUI) {
@@ -555,12 +565,13 @@ KDE_NO_EXPORT void KMPlayerTVSource::sync (bool fromUI) {
         m_app->hideBroadcastConfig ();
     if (fromUI) {
         tvdriver = m_configpage->driver->text ();
-        for (KMPlayer::NodePtr d=m_document->firstChild();d; d=d->nextSibling())
+        for (KMPlayer::Node *d=m_document->firstChild();d; d=d->nextSibling())
             if (d->id == id_node_tv_device)
-                static_cast <TVDevice *> (d.ptr ())->updateDevicePage ();
+                static_cast <TVDevice *> (d)->updateDevicePage ();
+        static_cast <KMPlayer::View*>(m_player->view ())->playList ()->updateTree (tree_id, m_document, 0, false, false);
     } else {
         m_configpage->driver->setText (tvdriver);
-        for (KMPlayer::NodePtr dp = m_document->firstChild (); dp; dp = dp->nextSibling ())
+        for (KMPlayer::Node *dp = m_document->firstChild (); dp; dp = dp->nextSibling ())
             if (dp->id == id_node_tv_device)
                 addTVDevicePage (KMPlayer::convertNode <TVDevice> (dp));
     }
@@ -582,9 +593,9 @@ KDE_NO_EXPORT QFrame * KMPlayerTVSource::prefPage (QWidget * parent) {
 }
 
 static bool hasTVDevice (KMPlayer::NodePtr doc, const QString & devstr) {
-    for (KMPlayer::NodePtr e = doc->firstChild (); e; e = e->nextSibling ())
+    for (KMPlayer::Node *e = doc->firstChild (); e; e = e->nextSibling ())
         if (e->id == id_node_tv_device &&
-                KMPlayer::convertNode <TVDevice> (e)->src == devstr)
+                static_cast <TVDevice *> (e)->src == devstr)
             return true;
     return false;
 }
@@ -606,6 +617,7 @@ KDE_NO_EXPORT void KMPlayerTVSource::slotScanFinished (TVDevice * tvdevice) {
     if (tvdevice) {
         tvdevice->zombie = false;
         addTVDevicePage (tvdevice, true);
+        static_cast <KMPlayer::View*>(m_player->view ())->playList ()->updateTree (tree_id, m_document, 0, false, false);
     } else
         KMessageBox::error(m_configpage,i18n("No device found."),i18n("Error"));
 }
@@ -614,7 +626,7 @@ KDE_NO_EXPORT void KMPlayerTVSource::addTVDevicePage(TVDevice *dev, bool show) {
     if (dev->device_page)
         dev->device_page->deleteLater ();
     dev->device_page = new TVDevicePage (m_configpage->notebook, dev);
-    m_configpage->notebook->insertTab (dev->device_page, dev->pretty_name);
+    m_configpage->notebook->insertTab (dev->device_page, dev->title);
     connect (dev->device_page, SIGNAL (deleted (TVDevicePage *)),
              this, SLOT (slotDeviceDeleted (TVDevicePage *)));
     if (show)
@@ -624,12 +636,17 @@ KDE_NO_EXPORT void KMPlayerTVSource::addTVDevicePage(TVDevice *dev, bool show) {
 KDE_NO_EXPORT void KMPlayerTVSource::slotDeviceDeleted (TVDevicePage *devpage) {
     m_document->removeChild (devpage->device_doc);
     m_configpage->notebook->setCurrentPage (0);
+    static_cast <KMPlayer::View*>(m_player->view ())->playList ()->updateTree (tree_id, m_document, 0, false, false);
 }
 
 //-----------------------------------------------------------------------------
 
 KDE_NO_CDTOR_EXPORT TVDeviceScannerSource::TVDeviceScannerSource (KMPlayerTVSource * src)
- : KMPlayer::Source (i18n ("TVScanner"), src->player (), "tvscanner"), m_tvsource (src), m_tvdevice (0L) {
+ : KMPlayer::Source (i18n ("TVScanner"), src->player (), "tvscanner"),
+   m_tvsource (src),
+   m_tvdevice (0L),
+   m_process (NULL),
+   m_viewer (NULL) {
 }
 
 KDE_NO_EXPORT void TVDeviceScannerSource::init () {
@@ -637,13 +654,13 @@ KDE_NO_EXPORT void TVDeviceScannerSource::init () {
 
 KDE_NO_EXPORT bool TVDeviceScannerSource::processOutput (const QString & line) {
     if (m_nameRegExp.search (line) > -1) {
-        m_tvdevice->pretty_name = m_nameRegExp.cap (1);
-        m_tvdevice->setAttribute(KMPlayer::StringPool::attr_name,m_tvdevice->pretty_name);
-        kdDebug() << "Name " << m_tvdevice->pretty_name << endl;
+        m_tvdevice->title = m_nameRegExp.cap (1);
+        m_tvdevice->setAttribute(KMPlayer::Ids::attr_name,m_tvdevice->title);
+        kDebug() << "Name " << m_tvdevice->title;
     } else if (m_sizesRegExp.search (line) > -1) {
-        m_tvdevice->setAttribute (KMPlayer::StringPool::attr_width,
+        m_tvdevice->setAttribute (KMPlayer::Ids::attr_width,
                 m_sizesRegExp.cap(1));
-        m_tvdevice->setAttribute (KMPlayer::StringPool::attr_height,
+        m_tvdevice->setAttribute (KMPlayer::Ids::attr_height,
                 m_sizesRegExp.cap(2));
         m_tvdevice->setAttribute ("minwidth", m_sizesRegExp.cap (1));
         m_tvdevice->setAttribute ("minheight", m_sizesRegExp.cap (2));
@@ -656,9 +673,27 @@ KDE_NO_EXPORT bool TVDeviceScannerSource::processOutput (const QString & line) {
         if (m_inputRegExp.cap (3).toInt () == 1)
             input->setAttribute ("tuner", "1");
         m_tvdevice->appendChild (input);
-        kdDebug() << "Input " << input->mrl ()->pretty_name << endl;
-    } else
+        kDebug() << "Input " << input->mrl ()->title;
+    } else if (m_inputRegExpV4l2.search (line) > -1) {
+        KMPlayer::NodePtr doc = m_tvsource->document ();
+        QStringList sl = m_inputRegExpV4l2.cap(1).split (QChar (';'));
+        const QStringList::iterator e = sl.end ();
+        for (QStringList::iterator i = sl.begin (); i != e; ++i) {
+            int pos = (*i).indexOf (QChar ('='));
+            if (pos > 0) {
+                int id = (*i).left (pos).trimmed ().toInt ();
+                TVInput *input = new TVInput(doc,(*i).mid(pos+1).trimmed(), id);
+                if (!id && m_caps.indexOf ("tuner") > -1)
+                    input->setAttribute ("tuner", "1");
+                m_tvdevice->appendChild (input);
+            }
+        }
+    } else {
+        int pos = line.indexOf ("Capabilites:");
+        if (pos > 0)
+            m_caps = line.mid (pos + 12);
         return false;
+    }
     return true;
 }
 
@@ -677,7 +712,7 @@ KDE_NO_EXPORT bool TVDeviceScannerSource::isSeekable () {
 KDE_NO_EXPORT bool TVDeviceScannerSource::scan (const QString & dev, const QString & dri) {
     if (m_tvdevice)
         return false;
-    setURL (KURL ("tv://"));
+    setUrl ("tv://");
     KMPlayer::NodePtr doc = m_tvsource->document ();
     m_tvdevice = new TVDevice (doc, dev);
     m_tvsource->document ()->appendChild (m_tvdevice);
@@ -686,7 +721,7 @@ KDE_NO_EXPORT bool TVDeviceScannerSource::scan (const QString & dev, const QStri
     m_old_source = m_tvsource->player ()->source ();
     m_tvsource->player ()->setSource (this);
     m_identified = true;
-    play ();
+    play (m_tvdevice);
     return true;
 }
 
@@ -694,45 +729,77 @@ KDE_NO_EXPORT void TVDeviceScannerSource::activate () {
     m_nameRegExp.setPattern ("Selected device:\\s*([^\\s].*)");
     m_sizesRegExp.setPattern ("Supported sizes:\\s*([0-9]+)x([0-9]+) => ([0-9]+)x([0-9]+)");
     m_inputRegExp.setPattern ("\\s*([0-9]+):\\s*([^:]+):[^\\(]*\\(tuner:([01]),\\s*norm:([^\\)]+)\\)");
+    m_inputRegExpV4l2.setPattern ("inputs:((?:\\s*[0-9]+\\s*=\\s*[^;]+;)+)");
 }
 
 KDE_NO_EXPORT void TVDeviceScannerSource::deactivate () {
-    kdDebug () << "TVDeviceScannerSource::deactivate" << endl;
+    kDebug () << "TVDeviceScannerSource::deactivate";
     if (m_tvdevice) {
         if (m_tvdevice->parentNode ())
             m_tvdevice->parentNode ()->removeChild (m_tvdevice);
         m_tvdevice = 0L;
+        delete m_process;
         emit scanFinished (m_tvdevice);
     }
 }
 
-KDE_NO_EXPORT void TVDeviceScannerSource::play () {
+KDE_NO_EXPORT void TVDeviceScannerSource::play (KMPlayer::Mrl *) {
     if (!m_tvdevice)
         return;
-    QString args;
-    args.sprintf ("tv:// -tv driver=%s:device=%s -identify -frames 0", m_driver.ascii (), m_tvdevice->src.ascii ());
+    m_options.sprintf ("tv:// -tv driver=%s:device=%s -identify -frames 0", m_driver.toAscii ().data (), m_tvdevice->src.toAscii ().data ());
     m_tvsource->player ()->stop ();
-    m_tvsource->player ()->process ()->initProcess (m_player->settings ()->defaultView ()->viewer ());
-    KMPlayer::Process *proc = m_tvsource->player ()->players () ["mplayer"];
-    proc->setSource (this);
-    if (!static_cast <KMPlayer::MPlayer *> (proc)->run (args.ascii()))
-        deactivate ();
+    KMPlayer::Node *n = new KMPlayer::SourceDocument (this, QString ());
+    setDocument (n, n);
+    m_process = m_player->mediaManager()->processInfos()["mplayer"]->create (m_player, this);
+    m_viewer = m_player->viewWidget ()->viewArea ()->createVideoWidget ();
+    m_process->ready ();
 }
 
-KDE_NO_EXPORT void TVDeviceScannerSource::stateChange (KMPlayer::Process * p, KMPlayer::Process::State os, KMPlayer::Process::State ns) {
-    if (m_tvdevice &&  // can be deactivated
-            ns == KMPlayer::Process::Ready && os > KMPlayer::Process::Ready) {
-        TVDevice * dev = 0L;
-        kdDebug () << "scanning done " << m_tvdevice->hasChildNodes () << endl;
-        if (!m_tvdevice->hasChildNodes ())
-            m_tvsource->document ()->removeChild (m_tvdevice);
-        else
-            dev = m_tvdevice;
-        m_tvdevice = 0L;
-        m_player->setSource (m_old_source);
-        emit scanFinished (dev);
+KDE_NO_EXPORT void TVDeviceScannerSource::scanningFinished () {
+    TVDevice * dev = 0L;
+    delete m_process;
+    kDebug () << "scanning done " << m_tvdevice->hasChildNodes ();
+    if (!m_tvdevice->hasChildNodes ()) {
+        m_tvsource->document ()->removeChild (m_tvdevice);
+    } else {
+        dev = m_tvdevice;
+        if (width () > 0 && height () > 0) {
+            m_tvdevice->setAttribute (KMPlayer::Ids::attr_width,
+                    QString::number (width ()));
+            m_tvdevice->setAttribute (KMPlayer::Ids::attr_height,
+                    QString::number (height ()));
+        }
     }
-    KMPlayer::Source::stateChange (p, os, ns);
+    m_tvdevice = 0L;
+    m_player->setSource (m_old_source);
+    emit scanFinished (dev);
 }
 
+void TVDeviceScannerSource::stateChange (KMPlayer::IProcess *,
+                   KMPlayer::IProcess::State os, KMPlayer::IProcess::State ns) {
+    if (KMPlayer::IProcess::Ready == ns) {
+        if (os > KMPlayer::IProcess::Ready)
+            QTimer::singleShot (0, this, SLOT (scanningFinished()));
+        else if (m_process && os < KMPlayer::IProcess::Ready)
+            m_process->play ();
+    }
+}
+
+void TVDeviceScannerSource::processDestroyed (KMPlayer::IProcess *) {
+    m_process = NULL;
+    KMPlayer::View *view = m_player->viewWidget ();
+    if (view)
+        view->viewArea ()->destroyVideoWidget (m_viewer);
+    m_viewer = NULL;
+}
+
+KMPlayer::IViewer *TVDeviceScannerSource::viewer () {
+    return m_viewer;
+}
+
+KMPlayer::Mrl *TVDeviceScannerSource::getMrl () {
+    return document ()->mrl ();
+}
+
+
 #include "kmplayertvsource.moc"
