@@ -20,13 +20,14 @@
 #ifndef _KMPLAYER_TV_SOURCE_H_
 #define _KMPLAYER_TV_SOURCE_H_
 
-#include <qguardedptr.h>
+#include <QPointer>
 #include <qstring.h>
 #include <qframe.h>
 
 #include "kmplayerappsource.h"
 #include "kmplayerconfig.h"
-#include "kmplayer.h"
+#include "mediaobject.h"
+#include "kmplayer_lists.h"
 
 const short id_node_tv_document = 40;
 const short id_node_tv_device = 41;
@@ -36,16 +37,11 @@ const short id_node_tv_channel = 43;
 class KMPlayerPrefSourcePageTV;         // source, TV
 class TVDeviceScannerSource;
 class KMPlayerTVSource;
-class KURLRequester;
-class KHistoryCombo;
+class KUrlRequester;
 class KMPlayerApp;
 class QTabWidget;
-class QGroupBox;
 class QLineEdit;
 class QCheckBox;
-class KComboBox;
-class KConfig;
-class QTable;
 
 
 class KMPLAYER_NO_EXPORT TVDevicePage : public QFrame {
@@ -55,7 +51,7 @@ public:
     KDE_NO_CDTOR_EXPORT ~TVDevicePage () {}
 
     QLineEdit * name;
-    KURLRequester * audiodevice;
+    KUrlRequester * audiodevice;
     QLineEdit * sizewidth;
     QLineEdit * sizeheight;
     QCheckBox * noplayback;
@@ -73,7 +69,7 @@ public:
     KMPlayerPrefSourcePageTV (QWidget *parent, KMPlayerTVSource *);
     KDE_NO_CDTOR_EXPORT ~KMPlayerPrefSourcePageTV () {}
     QLineEdit * driver;
-    KURLRequester * device;
+    KUrlRequester * device;
     QPushButton * scan;
     QTabWidget * notebook;
 protected:
@@ -83,7 +79,7 @@ protected:
 
 class KMPLAYER_NO_EXPORT TVNode : public KMPlayer::GenericMrl {
 public:
-    TVNode (KMPlayer::NodePtr &d, const QString &s, const char * t, short id, const QString &n=QString::null);
+    TVNode (KMPlayer::NodePtr &d, const QString &s, const char * t, short id, const QString &n=QString ());
     virtual void setNodeName (const QString &);
 };
 
@@ -106,7 +102,7 @@ public:
     TVInput (KMPlayer::NodePtr & d, const QString & n, int id);
     TVInput (KMPlayer::NodePtr & d);
     KDE_NO_CDTOR_EXPORT ~TVInput () {}
-    KMPlayer::NodePtr childFromTag (const QString &);
+    KMPlayer::Node *childFromTag (const QString &);
     void setNodeName (const QString &);
     void closed ();
 };
@@ -116,34 +112,36 @@ public:
  */
 class KMPLAYER_NO_EXPORT TVDevice : public TVNode {
 public:
-    TVDevice (KMPlayer::NodePtr & d, const QString & d);
+    TVDevice (KMPlayer::NodePtr & d, const QString & s);
     TVDevice (KMPlayer::NodePtr & d);
     ~TVDevice ();
-    KMPlayer::NodePtr childFromTag (const QString &);
+    KMPlayer::Node *childFromTag (const QString &);
     void closed ();
-    void childDone (KMPlayer::NodePtr child);
+    void message (KMPlayer::MessageType msg, void *content=NULL);
+    void *role (KMPlayer::RoleType msg, void *content=NULL);
     void setNodeName (const QString &);
-    bool expose () const { return false; }
     void updateNodeName ();
     void updateDevicePage ();
     bool zombie;
-    QGuardedPtr <TVDevicePage> device_page;
+    QPointer <TVDevicePage> device_page;
 };
 
 class KMPLAYER_NO_EXPORT TVDocument : public FileDocument {
     KMPlayerTVSource * m_source;
 public:
     TVDocument (KMPlayerTVSource *);
-    KMPlayer::NodePtr childFromTag (const QString &);
+    KMPlayer::Node *childFromTag (const QString &);
+    void defer ();
     KDE_NO_EXPORT const char * nodeName () const { return "tvdevices"; }
-    void childDone (KMPlayer::NodePtr child);
+    void message (KMPlayer::MessageType msg, void *content=NULL);
 };
 
 
 /*
  * Source form scanning TV devices
  */
-class KMPLAYER_NO_EXPORT TVDeviceScannerSource : public KMPlayer::Source {
+class KMPLAYER_NO_EXPORT TVDeviceScannerSource
+                             : public KMPlayer::Source, KMPlayer::ProcessUser {
     Q_OBJECT
 public:
     TVDeviceScannerSource (KMPlayerTVSource * src);
@@ -154,21 +152,32 @@ public:
     virtual bool hasLength ();
     virtual bool isSeekable ();
     virtual bool scan (const QString & device, const QString & driver);
+
+    virtual void starting (KMPlayer::IProcess *) {}
+    virtual void stateChange (KMPlayer::IProcess *, KMPlayer::IProcess::State, KMPlayer::IProcess::State);
+    virtual void processDestroyed (KMPlayer::IProcess *p);
+    virtual KMPlayer::IViewer *viewer ();
+    virtual KMPlayer::Mrl *getMrl ();
+
 public slots:
     virtual void activate ();
     virtual void deactivate ();
-    virtual void play ();
-    virtual void stateChange (KMPlayer::Process *, KMPlayer::Process::State os, KMPlayer::Process::State ns);
+    virtual void play (KMPlayer::Mrl *);
+    void scanningFinished ();
 signals:
     void scanFinished (TVDevice * tvdevice);
 private:
     KMPlayerTVSource * m_tvsource;
     TVDevice * m_tvdevice;
+    KMPlayer::IProcess *m_process;
+    KMPlayer::IViewer *m_viewer;
     KMPlayer::Source * m_old_source;
     QString m_driver;
+    QString m_caps;
     QRegExp m_nameRegExp;
     QRegExp m_sizesRegExp;
     QRegExp m_inputRegExp;
+    QRegExp m_inputRegExpV4l2;
 };
 
 /*
@@ -177,7 +186,7 @@ private:
 class KMPLAYER_NO_EXPORT KMPlayerTVSource : public KMPlayerMenuSource, public KMPlayer::PreferencesPage {
     Q_OBJECT
 public:
-    KMPlayerTVSource (KMPlayerApp * app, QPopupMenu * m);
+    KMPlayerTVSource (KMPlayerApp * app, QMenu * m);
     virtual ~KMPlayerTVSource ();
     virtual QString filterOptions ();
     virtual bool hasLength ();
@@ -185,17 +194,17 @@ public:
     virtual KMPlayer::NodePtr root ();
     void buildMenu ();
     virtual QString prettyName ();
-    virtual void write (KConfig *);
-    virtual void read (KConfig *);
+    virtual void write (KSharedConfigPtr);
+    virtual void read (KSharedConfigPtr);
     virtual void sync (bool);
     virtual void prefLocation (QString & item, QString & icon, QString & tab);
     virtual QFrame * prefPage (QWidget * parent);
     void readXML ();
+    void setCurrent (KMPlayer::Mrl *);
 public slots:
     virtual void activate ();
     virtual void deactivate ();
-    virtual void playCurrent ();
-    virtual void jump (KMPlayer::NodePtr e);
+    virtual void play (KMPlayer::Mrl *);
     void menuAboutToShow ();
     void menuClicked (int id);
 private slots:
@@ -204,10 +213,9 @@ private slots:
     void slotDeviceDeleted (TVDevicePage *);
 private:
     void addTVDevicePage (TVDevice * dev, bool show=false);
-    void buildArguments ();
     KMPlayer::NodePtrW m_cur_tvdevice;
     KMPlayer::NodePtrW m_cur_tvinput;
-    QPopupMenu * m_channelmenu;
+    QMenu * m_channelmenu;
     QString tvdriver;
     KMPlayerPrefSourcePageTV * m_configpage;
     TVDeviceScannerSource * scanner;
