@@ -19,43 +19,30 @@
 #ifndef KMPLAYERVIEW_H
 #define KMPLAYERVIEW_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif 
+#include "config-kmplayer.h"
 #include <qwidget.h>
-#include <qguardedptr.h>
 #include <qtextedit.h>
+#include <QImage>
 
-#include <kdockwidget.h>
 #include <kurl.h>
-#include <qxembed.h>
 #include <kmediaplayer/view.h>
 
 #include "kmplayersource.h"
 
 #define MOUSE_INVISIBLE_DELAY 2000
 
-class QWidgetStack;
-class QPixmap;
-class QPaintDevice;
-class QPainter;
-class QPopupMenu;
 class QSlider;
-class QLabel;
-class QAccel;
+class QMainWindow;
+class QDockWidget;
 class KActionCollection;
 class KAction;
-class KShortcut;
 class KStatusBar;
-class KFindDialog;
 
 namespace KMPlayer {
 
 class View;
 class ViewArea;
-class Viewer;
 class ControlPanel;
-class VolumeBar;
 class Console;
 class PlayListView;
 class PlayListView;
@@ -88,6 +75,16 @@ private:
     View * m_view;
 };
 
+class KMPLAYER_NO_EXPORT PictureWidget : public QWidget {
+    View * m_view;
+public:
+    PictureWidget (QWidget * parent, View * view);
+    KDE_NO_CDTOR_EXPORT ~PictureWidget () {}
+protected:
+    void mousePressEvent (QMouseEvent *);
+    void mouseMoveEvent (QMouseEvent *);
+};
+
 /*
  * The view containing ViewArea and playlist
  */
@@ -100,30 +97,27 @@ public:
     enum StatusBarMode {
         SB_Hide, SB_Show, SB_Only /* no video widget */
     };
-    enum WidgetType {
-        WT_Video, WT_Console, WT_Picture, WT_Last
-    };
 
-    View (QWidget *parent, const char *);
+    View (QWidget *parent);
     ~View();
 
     void addText (const QString &, bool eol=false);
-    void init (KActionCollection * ac);
+    void init (KActionCollection * ac, bool transparent);
+    void initDock (QWidget *central);
     void reset ();
     //void print(QPrinter *pPrinter);
 
     TextEdit * console () const { return m_multiedit; }
-    KDE_NO_EXPORT Viewer * viewer () const { return m_viewer; }
+    PictureWidget *picture () const KMPLAYER_NO_MBR_EXPORT { return m_picture; }
     KDE_NO_EXPORT ControlPanel * controlPanel () const {return m_control_panel;}
     KDE_NO_EXPORT StatusBar * statusBar () const {return m_status_bar;}
     KDE_NO_EXPORT PlayListView * playList () const { return m_playlist; }
     KDE_NO_EXPORT InfoWindow * infoPanel () const { return m_infopanel; }
-    KDE_NO_EXPORT QWidgetStack * widgetStack () const { return m_widgetstack; }
-    KDE_NO_EXPORT KDockArea * docArea () const { return m_dockarea; }
+    KDE_NO_EXPORT QMainWindow *dockArea () const { return m_dockarea; }
+    KDE_NO_EXPORT QDockWidget *dockPlaylist () const { return m_dock_playlist; }
     KDE_NO_EXPORT ViewArea * viewArea () const { return m_view_area; }
     KDE_NO_EXPORT bool keepSizeRatio () const { return m_keepsizeratio; }
     void setKeepSizeRatio (bool b);
-    void showWidget (WidgetType w);
     void setControlPanelMode (ControlPanelMode m);
     void setStatusBarMode (StatusBarMode m);
     void setEraseColor (const QColor &);
@@ -131,15 +125,13 @@ public:
     KDE_NO_EXPORT StatusBarMode statusBarMode () const { return m_statusbar_mode; }
     void delayedShowButtons (bool show);
     bool isFullScreen () const;
-    int statusBarHeight () const;
+    int statusBarHeight () const KMPLAYER_NO_MBR_EXPORT;
     KDE_NO_EXPORT bool editMode () const { return m_edit_mode; }
+#ifndef KMPLAYER_WITH_CAIRO
     bool setPicture (const QString & path);
-    KDE_NO_EXPORT QPixmap * image () const { return m_image; }
+#endif
     void setNoInfoMessages (bool b) { m_no_info = b; }
     void setViewOnly ();
-    void setInfoPanelOnly ();
-    void setPlaylistOnly ();
-    bool playlistVisible () const { return !m_dock_playlist->mayBeShow (); }
     void setEditMode (RootPlayListItem *, bool enable=true);
     void dragEnterEvent (QDragEnterEvent *);
     void dropEvent (QDropEvent *);
@@ -149,9 +141,8 @@ public:
     void playingStart ();
     /* shows panel */
     void playingStop ();
+    void mouseMoved (int x, int y) KMPLAYER_NO_MBR_EXPORT;
 public slots:
-    void setVolume (int);
-    void updateVolume ();
     void fullScreen ();
     void updateLayout ();
     void toggleShowPlaylist ();
@@ -159,48 +150,38 @@ public slots:
     void setInfoMessage (const QString & msg);
     void setStatusMessage (const QString & msg);
 signals:
-    void urlDropped (const KURL::List & urls);
+    void urlDropped (const KUrl::List & urls);
     void pictureClicked ();
     void fullScreenChanged ();
-    void windowVideoConsoleToggled (int wt);
+    void windowVideoConsoleToggled (bool show);
 protected:
-    void leaveEvent (QEvent *);
-    void timerEvent (QTimerEvent *);
-    bool x11Event (XEvent *);
+    void leaveEvent (QEvent *) KDE_NO_EXPORT;
+    void timerEvent (QTimerEvent *) KDE_NO_EXPORT;
 private:
-    // widget for player's output
-    Viewer * m_viewer;
     // console output
     TextEdit * m_multiedit;
-    // widget stack contains m_viewer, m_multiedit and m_picturewidget
-    QWidgetStack * m_widgetstack;
+    PictureWidget *m_picture;
     // widget that layouts m_widgetstack for ratio setting and m_control_panel
     ViewArea * m_view_area;
     // playlist widget
     PlayListView * m_playlist;
     // infopanel widget
     InfoWindow * m_infopanel;
-    // all widget types
-    QWidget * m_widgettypes [WT_Last];
-    KDockArea * m_dockarea;
-    KDockWidget * m_dock_video;
-    KDockWidget * m_dock_playlist;
-    KDockWidget * m_dock_infopanel;
+    QMainWindow *m_dockarea;
+    QDockWidget *m_dock_playlist;
+    QDockWidget *m_dock_infopanel;
     QString tmplog;
-    QPixmap * m_image;
+    QImage m_image;
     ControlPanel * m_control_panel;
     StatusBar * m_status_bar;
-    QSlider * m_volume_slider;
-    const char * m_mixer_object;
     ControlPanelMode m_controlpanel_mode;
     ControlPanelMode m_old_controlpanel_mode;
     StatusBarMode m_statusbar_mode;
     int controlbar_timer;
     int infopanel_timer;
+    int m_powerManagerStopSleep;
     bool m_keepsizeratio;
     bool m_playing;
-    bool m_mixer_init;
-    bool m_inVolumeUpdate;
     bool m_sreensaver_disabled;
     bool m_tmplog_needs_eol;
     bool m_revert_fullscreen;
@@ -208,37 +189,6 @@ private:
     bool m_edit_mode;
 };
 
-/*
- * The video widget
- */
-class KMPLAYER_EXPORT Viewer : public QXEmbed {
-    Q_OBJECT
-public:
-    Viewer(QWidget *parent, View * view);
-    ~Viewer();
-
-    int heightForWidth (int w) const;
-
-    void setAspect (float a);
-    float aspect () { return m_aspect; }
-    void sendKeyEvent (int key);
-    void setBackgroundColor (const QColor & c);
-    void resetBackgroundColor ();
-    void setCurrentBackgroundColor (const QColor & c);
-    KDE_NO_EXPORT View * view () const { return m_view; }
-public slots:
-    void sendConfigureEvent ();
-protected:
-    void dragEnterEvent (QDragEnterEvent *);
-    void dropEvent (QDropEvent *);
-    void mouseMoveEvent (QMouseEvent * e);
-    void contextMenuEvent (QContextMenuEvent * e);
-private:
-    unsigned int m_bgcolor;
-    float m_aspect;
-    View * m_view;
-};
-
 } // namespace
 
 #endif // KMPLAYERVIEW_H
