@@ -125,7 +125,7 @@ void ImageData::copyImage (Surface *s, const SSize &sz, cairo_surface_t *similar
     if (zoom) {
         cairo_matrix_t mat;
         Single zx, zy, zw, zh;
-        zoom->calcSizes (NULL, width, height, zx, zy, zw, zh);
+        zoom->calcSizes (NULL, NULL, width, height, zx, zy, zw, zh);
         cairo_matrix_init_translate (&mat, zx, zy);
         cairo_matrix_scale (&mat, 1.0 * zw/w, 1.0 * zh/h);
         cairo_pattern_set_matrix (img_pat, &mat);
@@ -178,15 +178,18 @@ class KMPLAYER_NO_EXPORT CairoPaintVisitor : public Visitor {
     cairo_surface_t * cairo_surface;
     Matrix matrix;
     // stack vars need for transitions
-    SMIL::MediaType *cur_media;
+    TransitionModule *cur_transition;
     cairo_pattern_t * cur_pat;
     cairo_matrix_t cur_mat;
+    SMIL::RegionBase::BackgroundRepeat bg_repeat;
+    ImageData *bg_image;
     float opacity;
     bool toplevel;
 
     void traverseRegion (Node *reg, Surface *s);
     void updateExternal (SMIL::MediaType *av, SurfacePtr s);
-    void paint(SMIL::MediaType *, Surface *, const IPoint &p, const IRect &);
+    void paint (TransitionModule *trans, MediaOpacity mopacity, Surface *s,
+                const IPoint &p, const IRect &);
     void video (Mrl *mt, Surface *s);
 public:
     cairo_t * cr;
@@ -217,7 +220,9 @@ KDE_NO_CDTOR_EXPORT
 CairoPaintVisitor::CairoPaintVisitor (cairo_surface_t * cs, Matrix m,
         const IRect & rect, QColor c, bool top)
  : clip (rect), cairo_surface (cs),
-   matrix (m), toplevel (top) {
+   matrix (m),
+   bg_repeat (SMIL::RegionBase::BgRepeat), bg_image (NULL),
+   toplevel (top) {
     cr = cairo_create (cs);
     if (toplevel) {
         cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
@@ -278,33 +283,8 @@ KDE_NO_EXPORT void CairoPaintVisitor::traverseRegion (Node *node, Surface *s) {
 }
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Layout *layout) {
-    //kDebug() << "Visit " << layout->nodeName();
-    if (layout->root_layout) {
-        Surface *s = (Surface *)layout->root_layout->role (RoleDisplay);
-        if (s) {
-            //cairo_save (cr);
-            Matrix m = matrix;
-
-            IRect scr = matrix.toScreen (SRect (0, 0, s->bounds.size));
-
-            IRect clip_save = clip;
-            clip = clip.intersect (scr);
-
-            if (s->background_color & 0xff000000) {
-                CAIRO_SET_SOURCE_RGB (cr, s->background_color);
-                cairo_rectangle (cr,
-                        clip.x (), clip.y (), clip.width (), clip.height ());
-                cairo_fill (cr);
-            }
-
-            matrix = Matrix (0, 0, s->xscale, s->yscale);
-            matrix.transform (m);
-            traverseRegion (layout->root_layout, s);
-            //cairo_restore (cr);
-            matrix = m;
-            clip = clip_save;
-        }
-    }
+    if (layout->root_layout)
+        layout->root_layout->accept (this);
 }
 
 
@@ -324,10 +304,13 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RegionBase *reg) {
         if (clip.intersect (scr).isEmpty ())
             return;
         Matrix m = matrix;
-        matrix = Matrix (rect.x(), rect.y(), 1.0, 1.0);
+        matrix = Matrix (rect.x(), rect.y(), s->xscale, s->yscale);
         matrix.transform (m);
         IRect clip_save = clip;
         clip = clip.intersect (scr);
+        SMIL::RegionBase::BackgroundRepeat bg_repeat_save = bg_repeat;
+        if (SMIL::RegionBase::BgInherit != reg->bg_repeat)
+            bg_repeat = reg->bg_repeat;
         cairo_save (cr);
 
         Surface *cs = s->firstChild ();
@@ -337,7 +320,13 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RegionBase *reg) {
         ImageMedia *im = reg->media_info
             ? (ImageMedia *) reg->media_info->media
             : NULL;
+
+        ImageData *bg_save = bg_image;
         ImageData *bg_img = im && !im->isEmpty() ? im->cached_img.ptr () : NULL;
+        if (reg->background_image == "inherit")
+            bg_img = bg_image;
+        else
+            bg_image = bg_img;
         unsigned int bg_alpha = s->background_color & 0xff000000;
         if ((SMIL::RegionBase::ShowAlways == reg->show_background ||
                     reg->m_AttachedMediaTypes.first ()) &&
@@ -370,8 +359,27 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RegionBase *reg) {
                 cairo_matrix_init_translate (&mat, -scr.x (), -scr.y ());
                 cairo_pattern_set_matrix (pat, &mat);
                 cairo_set_source (cr, pat);
-                cairo_rectangle (cr,
-                        clip.x (), clip.y (), clip.width (), clip.height ());
+                int cw = clip.width ();
+                int ch = clip.height ();
+                switch (bg_repeat) {
+                case SMIL::RegionBase::BgRepeatX:
+                    if (h < ch)
+                        ch = h;
+                    break;
+                case SMIL::RegionBase::BgRepeatY:
+                    if (w < cw)
+                        cw = w;
+                    break;
+                case SMIL::RegionBase::BgNoRepeat:
+                    if (w < cw)
+                        cw = w;
+                    if (h < ch)
+                        ch = h;
+                    break;
+                default:
+                    break;
+                }
+                cairo_rectangle (cr, clip.x (), clip.y (), cw, ch);
                 cairo_fill (cr);
                 cairo_pattern_destroy (pat);
                 if (bg_img->has_alpha)
@@ -457,7 +465,9 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RegionBase *reg) {
             }
         }
         cairo_restore (cr);
+        bg_image = bg_save;
         matrix = m;
+        bg_repeat = bg_repeat_save;
         clip = clip_save;
         s->dirty = false;
     }
@@ -472,8 +482,8 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RegionBase *reg) {
     }
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Transition *trans) {
-    float perc = trans->start_progress + (trans->end_progress - trans->start_progress)*cur_media->trans_gain;
-    if (cur_media->trans_out_active)
+    float perc = trans->start_progress + (trans->end_progress - trans->start_progress)*cur_transition->trans_gain;
+    if (cur_transition->trans_out_active)
         perc = 1.0 - perc;
     if (SMIL::Transition::Fade == trans->type) {
         CAIRO_SET_PATTERN_COND(cr, cur_pat, cur_mat)
@@ -649,17 +659,18 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RefMediaType *ref) {
         video (ref, s);
 }
 
-KDE_NO_EXPORT void CairoPaintVisitor::paint (SMIL::MediaType *mt, Surface *s,
+KDE_NO_EXPORT void CairoPaintVisitor::paint (TransitionModule *trans,
+        MediaOpacity mopacity, Surface *s,
         const IPoint &point, const IRect &rect) {
     cairo_save (cr);
     opacity = 1.0;
     cairo_matrix_init_translate (&cur_mat, -point.x, -point.y);
     cur_pat = cairo_pattern_create_for_surface (s->surface);
-    if (mt->active_trans) {
+    if (trans->active_trans) {
         IRect clip_save = clip;
         clip = rect;
-        cur_media = mt;
-        mt->active_trans->accept (this);
+        cur_transition = trans;
+        trans->active_trans->accept (this);
         clip = clip_save;
     } else {
         cairo_pattern_set_extend (cur_pat, CAIRO_EXTEND_NONE);
@@ -668,7 +679,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::paint (SMIL::MediaType *mt, Surface *s,
         cairo_set_source (cr, cur_pat);
         cairo_rectangle (cr, rect.x(), rect.y(), rect.width(), rect.height());
     }
-    opacity *= mt->opacity / 100.0;
+    opacity *= mopacity.opacity / 100.0;
     bool over = opacity < 0.99 ||
                 CAIRO_CONTENT_COLOR != cairo_surface_get_content (s->surface);
     cairo_operator_t op;
@@ -726,6 +737,7 @@ void CairoPaintVisitor::updateExternal (SMIL::MediaType *av, SurfacePtr s) {
     if (!s->surface || s->dirty) {
         Matrix m = matrix;
         m.translate (-scr.x (), -scr.y ());
+        m.scale (s->xscale, s->yscale);
         IRect r (clip_rect.x() - scr.x () - 1, clip_rect.y() - scr.y () - 1,
                 clip_rect.width() + 3, clip_rect.height() + 3);
         if (!s->surface) {
@@ -737,11 +749,10 @@ void CairoPaintVisitor::updateExternal (SMIL::MediaType *av, SurfacePtr s) {
         ext_mrl->accept (&visitor);
         s->dirty = false;
     }
-    paint (av, s.ptr (), scr.point, clip_rect);
+    paint (&av->transition, av->media_opacity, s.ptr (), scr.point, clip_rect);
 }
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::ImageMediaType * img) {
-    //kDebug() << "Visit " << img->nodeName() << " " << img->src;
     if (!img->media_info)
         return;
     Surface *s = img->surface ();
@@ -769,7 +780,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::ImageMediaType * img) {
     }
     if (!s->surface || s->dirty)
         id->copyImage (s, SSize (scr.width (), scr.height ()), cairo_surface, img->pan_zoom);
-    paint (img, s, scr.point, clip_rect);
+    paint (&img->transition, img->media_opacity, s, scr.point, clip_rect);
     s->dirty = false;
 }
 
@@ -798,6 +809,30 @@ static void calculateTextDimensions (PangoFontDescription *desc,
     cairo_surface_destroy (img_surf);
 }
 
+static cairo_t *createContext (cairo_surface_t *similar, Surface *s, int w, int h) {
+    unsigned int bg_alpha = s->background_color & 0xff000000;
+    bool clear = s->surface;
+    if (!s->surface)
+        s->surface = cairo_surface_create_similar (similar,
+                bg_alpha < 0xff000000
+                ? CAIRO_CONTENT_COLOR_ALPHA
+                : CAIRO_CONTENT_COLOR,
+                w, h);
+    cairo_t *cr = cairo_create (s->surface);
+    if (clear)
+        clearSurface (cr, IRect (0, 0, w, h));
+    cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
+
+    if (bg_alpha) {
+        if (bg_alpha < 0xff000000)
+            CAIRO_SET_SOURCE_ARGB (cr, s->background_color);
+        else
+            CAIRO_SET_SOURCE_RGB (cr, s->background_color);
+        cairo_paint (cr);
+    }
+    return cr;
+}
+
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::TextMediaType * txt) {
     if (!txt->media_info || !txt->media_info->media)
         return;
@@ -828,25 +863,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::TextMediaType * txt) {
             calculateTextDimensions (desc, text.data (),
                     w, 2 * ft_size, &pxw, &pxh, false);
         }
-        unsigned int bg_alpha = txt->background_color & 0xff000000;
-        if (!s->surface)
-            s->surface = cairo_surface_create_similar (cairo_surface,
-                    bg_alpha < 0xff000000
-                    ? CAIRO_CONTENT_COLOR_ALPHA
-                    : CAIRO_CONTENT_COLOR,
-                    pxw, pxh);
-        cairo_t *cr_txt = cairo_create (s->surface);
-        if (clear)
-            clearSurface (cr_txt, IRect (0, 0, pxw, pxh));
-        cairo_set_operator (cr_txt, CAIRO_OPERATOR_SOURCE);
-
-        if (bg_alpha) {
-            if (bg_alpha < 0xff000000)
-                CAIRO_SET_SOURCE_ARGB (cr_txt, txt->background_color);
-            else
-                CAIRO_SET_SOURCE_RGB (cr_txt, txt->background_color);
-            cairo_paint (cr_txt);
-        }
+        cairo_t *cr_txt = createContext (cairo_surface, s, pxw, pxh);
 
         CAIRO_SET_SOURCE_RGB (cr_txt, txt->font_color);
         PangoLayout *layout = pango_cairo_create_layout (cr_txt);
@@ -871,12 +888,11 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::TextMediaType * txt) {
     }
     IRect clip_rect = clip.intersect (scr);
     if (!clip_rect.isEmpty ())
-        paint (txt, s, scr.point, clip_rect);
+        paint (&txt->transition, txt->media_opacity, s, scr.point, clip_rect);
     s->dirty = false;
 }
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Brush * brush) {
-    //kDebug() << "Visit " << brush->nodeName();
     Surface *s = brush->surface ();
     if (s) {
         opacity = 1.0;
@@ -884,15 +900,15 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Brush * brush) {
         if (clip_rect.isEmpty ())
             return;
         cairo_save (cr);
-        if (brush->active_trans) {
-            cur_media = brush;
+        if (brush->transition.active_trans) {
+            cur_transition = &brush->transition;
             cur_pat = NULL;
-            brush->active_trans->accept (this);
+            brush->transition.active_trans->accept (this);
         } else {
             cairo_rectangle (cr, clip_rect.x (), clip_rect.y (),
                     clip_rect.width (), clip_rect.height ());
         }
-        opacity *= brush->opacity / 100.0;
+        opacity *= brush->media_opacity.opacity / 100.0;
         if (opacity < 0.99) {
             cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
             cairo_set_source_rgba (cr,
@@ -1062,6 +1078,7 @@ void SmilTextVisitor::visit (TextNode *text) {
 void SmilTextVisitor::visit (SMIL::TextFlow *flow) {
     if (flow->firstChild ()) {
         bool new_block = SMIL::id_node_p == flow->id ||
+            SMIL::id_node_br == flow->id ||
             SMIL::id_node_div == flow->id;
         float fs = info.props.font_size;
         if (fs < 0)
@@ -1079,8 +1096,6 @@ void SmilTextVisitor::visit (SMIL::TextFlow *flow) {
             max_font_size = info.props.font_size;
         info.span (scale);
 
-        SmilTextBlock *block = last;
-
         flow->firstChild ()->accept (this);
 
         if (rich_text.isEmpty ())
@@ -1114,11 +1129,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::SmilText *txt) {
 
         info.push ();
         if (info.first) {
-
-            s->surface = cairo_surface_create_similar (cairo_surface,
-                    CAIRO_CONTENT_COLOR_ALPHA, (int) w, info.voffset);
-            cairo_t *cr_txt = cairo_create (s->surface);
-            cairo_set_operator (cr_txt, CAIRO_OPERATOR_SOURCE);
+            cairo_t *cr_txt = createContext (cairo_surface, s, (int) w, info.voffset);
 
             CAIRO_SET_SOURCE_RGB (cr_txt, 0);
             SmilTextBlock *b = info.first;
@@ -1153,22 +1164,8 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::SmilText *txt) {
         }
     }
     IRect clip_rect = clip.intersect (scr);
-    if (!clip_rect.isEmpty ()) {
-        cairo_save (cr);
-        cairo_matrix_init_translate (&cur_mat, -scr.x (), -scr.y ());
-        cairo_pattern_t *pat = cairo_pattern_create_for_surface (s->surface);
-        cairo_pattern_set_extend (pat, CAIRO_EXTEND_NONE);
-        cairo_pattern_set_matrix (pat, &cur_mat);
-        cairo_pattern_set_filter (pat, CAIRO_FILTER_FAST);
-        cairo_set_source (cr, pat);
-        cairo_rectangle(cr, clip_rect.x (), clip_rect.y (),
-                clip_rect.width (), clip_rect.height ());
-        cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
-        cairo_fill (cr);
-        cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
-        cairo_pattern_destroy (pat);
-        cairo_restore (cr);
-    }
+    if (!clip_rect.isEmpty ())
+        paint (&txt->transition, txt->media_opacity, s, scr.point, clip_rect);
     s->dirty = false;
 }
 
@@ -1222,7 +1219,6 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Imfl * imfl) {
 }
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Fill * fi) {
-    //kDebug() << "Visit " << fi->nodeName();
     CAIRO_SET_SOURCE_RGB (cr, fi->color);
     if ((int)fi->w && (int)fi->h) {
         cairo_rectangle (cr, fi->x, fi->y, fi->w, fi->h);
@@ -1231,7 +1227,6 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Fill * fi) {
 }
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Fadein * fi) {
-    //kDebug() << "Visit " << fi->nodeName();
     if (fi->target && fi->target->id == RP::id_node_image) {
         RP::Image *img = convertNode <RP::Image> (fi->target);
         ImageMedia *im = img && img->media_info
@@ -1270,7 +1265,6 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Fadein * fi) {
 }
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Fadeout * fo) {
-    //kDebug() << "Visit " << fo->nodeName();
     if (fo->progress > 0) {
         CAIRO_SET_SOURCE_RGB (cr, fo->to_color);
         if ((int)fo->w && (int)fo->h) {
@@ -1284,7 +1278,6 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Fadeout * fo) {
 }
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Crossfade * cf) {
-    //kDebug() << "Visit " << cf->nodeName();
     if (cf->target && cf->target->id == RP::id_node_image) {
         RP::Image *img = convertNode <RP::Image> (cf->target);
         ImageMedia *im = img && img->media_info
@@ -1323,7 +1316,6 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Crossfade * cf) {
 }
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Wipe * wipe) {
-    //kDebug() << "Visit " << wipe->nodeName();
     if (wipe->target && wipe->target->id == RP::id_node_image) {
         RP::Image *img = convertNode <RP::Image> (wipe->target);
         ImageMedia *im = img && img->media_info
@@ -1382,7 +1374,6 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Wipe * wipe) {
 }
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::ViewChange * vc) {
-    //kDebug() << "Visit " << vc->nodeName();
     if (vc->unfinished () || vc->progress < 100) {
         cairo_pattern_t * pat = cairo_pop_group (cr); // from imfl
         cairo_pattern_set_extend (pat, CAIRO_EXTEND_NONE);
@@ -1425,9 +1416,9 @@ class KMPLAYER_NO_EXPORT MouseVisitor : public Visitor {
     bool bubble_up;
 
     bool deliverAndForward (Node *n, Surface *s, bool inside, bool deliver);
-    bool surfaceEvent (Node *mt, Surface *s);
+    void surfaceEvent (Node *mt, Surface *s);
 public:
-    MouseVisitor (ViewArea *v, MessageType evt, int x, int y);
+    MouseVisitor (ViewArea *v, MessageType evt, Matrix m, int x, int y);
     KDE_NO_CDTOR_EXPORT ~MouseVisitor () {}
     using Visitor::visit;
     void visit (Node * n);
@@ -1445,8 +1436,8 @@ public:
 } // namespace
 
 KDE_NO_CDTOR_EXPORT
-MouseVisitor::MouseVisitor (ViewArea *v, MessageType evt, int a, int b)
-  : view_area (v), event (evt), x (a), y (b),
+MouseVisitor::MouseVisitor (ViewArea *v, MessageType evt, Matrix m, int a, int b)
+  : view_area (v), matrix (m), event (evt), x (a), y (b),
     handled (false), bubble_up (false) {
 }
 
@@ -1460,25 +1451,8 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Smil *s) {
 }
 
 KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Layout * layout) {
-    Surface *s = (Surface *) layout->root_layout->role (RoleDisplay);
-    if (s) {
-        Matrix m = matrix;
-        SRect rect = s->bounds;
-        matrix = Matrix (rect.x(), rect.y(), s->xscale, s->yscale);
-        matrix.transform (m);
-
-        NodePtr node_save = source;
-        source = layout;
-        for (NodePtr r = layout->firstChild (); r; r = r->nextSibling ()) {
-            if (r->id == SMIL::id_node_region)
-                r->accept (this);
-            if (!source || !source->active ())
-                break;
-        }
-        source = node_save;
-
-        matrix = m;
-    }
+    if (layout->root_layout)
+        layout->root_layout->accept (this);
 }
 
 KDE_NO_EXPORT void MouseVisitor::visit (SMIL::RegionBase *region) {
@@ -1521,6 +1495,8 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::RegionBase *region) {
             return;
         }
 
+        NodePtrW src = source;
+        source = region;
         Matrix m = matrix;
         matrix = Matrix (rect.x(), rect.y(), 1.0, 1.0);
         matrix.transform (m);
@@ -1530,11 +1506,15 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::RegionBase *region) {
 
         bool child_handled = false;
         if (inside || s->has_mouse)
-            for (NodePtr r = region->firstChild (); r; r = r->nextSibling ()) {
-                r->accept (this);
-                child_handled |= handled;
-                if (!source || !source->active ())
+            for (SurfacePtr c = s->firstChild (); c; c = c->nextSibling ()) {
+                if (c->node && c->node->id == SMIL::id_node_region) {
+                    c->node->accept (this);
+                    child_handled |= handled;
+                    if (!source || !source->active ())
+                        break;
+                } else {
                     break;
+                }
             }
         child_handled &= !bubble_up;
         bubble_up = false;
@@ -1543,6 +1523,7 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::RegionBase *region) {
 
         handled = inside;
         matrix = m;
+        source = src;
     }
 }
 
@@ -1563,7 +1544,7 @@ static void followLink (SMIL::LinkingBase * link) {
             for (NodePtr p = link->parentNode (); p; p = p->parentNode ()) {
                 if (n->mrl () && n->mrl ()->opener == p) {
                     p->setState (Node::state_deferred);
-                    p->mrl ()->setParam (StringPool::attr_src, link->href, 0L);
+                    p->mrl ()->setParam (Ids::attr_src, link->href, 0L);
                     p->activate ();
                     break;
                 }
@@ -1645,6 +1626,9 @@ bool MouseVisitor::deliverAndForward (Node *node, Surface *s, bool inside, bool
     }
     s->has_mouse = inside;
 
+    if (event != MsgEventPointerMoved && !inside)
+        return false;
+
     NodePtrW node_save = node;
 
     if (forward) {
@@ -1673,33 +1657,35 @@ bool MouseVisitor::deliverAndForward (Node *node, Surface *s, bool inside, bool
     return true;
 }
 
-bool MouseVisitor::surfaceEvent (Node *node, Surface *s) {
+void MouseVisitor::surfaceEvent (Node *node, Surface *s) {
     if (!s)
-        return false;
+        return;
     if (s->node && s->node.ptr () != node) {
         s->node->accept (this);
-        return false;
+        return;
     }
     SRect rect = s->bounds;
     IRect scr = matrix.toScreen (rect);
     int rx = scr.x(), ry = scr.y(), rw = scr.width(), rh = scr.height();
     const bool inside = x > rx && x < rx+rw && y > ry && y< ry+rh;
-    return deliverAndForward (node, s, inside, true);
+    const bool had_mouse = s->has_mouse;
+    s->has_mouse = inside;
+    if (deliverAndForward (node, s, inside, true) &&
+            (inside || had_mouse) &&
+            s->firstChild () && s->firstChild ()->node) {
+        Matrix m = matrix;
+        matrix = Matrix (rect.x(), rect.y(), s->xscale, s->yscale);
+        matrix.transform (m);
+        s->firstChild ()->node->accept (this);
+        matrix = m;
+    }
 }
 
 KDE_NO_EXPORT void MouseVisitor::visit (SMIL::MediaType *mt) {
-    if (mt->sensitivity == SMIL::MediaType::sens_transparent) {
+    if (mt->sensitivity == SMIL::MediaType::sens_transparent)
         bubble_up = true;
-        return;
-    }
-    if (!surfaceEvent (mt, mt->surface ()))
-        return;
-
-    SMIL::RegionBase *r=convertNode<SMIL::RegionBase>(mt->region_node);
-    if (r && r->role (RoleDisplay) &&
-            r->id != SMIL::id_node_smil &&
-            r->region_surface->node && r != r->region_surface->node.ptr ())
-        return r->region_surface->node->accept (this);
+    else
+        surfaceEvent (mt, mt->surface ());
 }
 
 KDE_NO_EXPORT void MouseVisitor::visit (SMIL::SmilText *st) {
@@ -1712,22 +1698,29 @@ namespace KMPlayer {
 class KMPLAYER_NO_EXPORT ViewerAreaPrivate {
 public:
     ViewerAreaPrivate (ViewArea *v)
-        : m_view_area (v), backing_store (0), width (0), height (0) {
+        : m_view_area (v), backing_store (0),
+          width (0), height (0), have_gc (false) {
     }
     ~ViewerAreaPrivate() {
         destroyBackingStore ();
+        if (have_gc)
+            XFreeGC (QX11Info::display (), gc);
+    }
+    void clearSurface (Surface *s) {
+#ifdef KMPLAYER_WITH_CAIRO
+        if (s->surface) {
+            cairo_surface_destroy (s->surface);
+            s->surface = 0L;
+        }
+        destroyBackingStore ();
+#endif
     }
     void resizeSurface (Surface *s) {
 #ifdef KMPLAYER_WITH_CAIRO
         int w = m_view_area->width ();
         int h = m_view_area->height ();
         if ((w != width || h != height) && s->surface) {
-            Display *d = QX11Info::display();
-            //cairo_xlib_surface_set_size (s->surface, w, h);
-            destroyBackingStore ();
-            backing_store = XCreatePixmap (d, m_view_area->winId (),
-                    w, h, QX11Info::appDepth ());
-            cairo_xlib_surface_set_drawable(s->surface, backing_store, w,h);
+            clearSurface (s);
             width = w;
             height = h;
         }
@@ -1753,10 +1746,19 @@ public:
             w, h);*/
     }
     void swapBuffer (const IRect &sr, int dx, int dy) {
-        GC gc = XCreateGC (QX11Info::display(), backing_store, 0, NULL);
+        if (!have_gc) {
+            XGCValues values;
+            values.graphics_exposures = false;
+            values.function = GXcopy;
+            values.fill_style = FillSolid;
+            values.subwindow_mode = ClipByChildren;
+            gc = XCreateGC (QX11Info::display (), backing_store,
+                    GCGraphicsExposures | GCFunction |
+                    GCFillStyle | GCSubwindowMode, &values);
+            have_gc = true;
+        }
         XCopyArea (QX11Info::display(), backing_store, m_view_area->winId(),
                 gc, sr.x(), sr.y(), sr.width (), sr.height (), dx, dy);
-        XFreeGC (QX11Info::display(), gc);
         XFlush (QX11Info::display());
     }
 #endif
@@ -1769,8 +1771,10 @@ public:
     }
     ViewArea *m_view_area;
     Drawable backing_store;
+    GC gc;
     int width;
     int height;
+    bool have_gc;
 };
 
 class KMPLAYER_NO_EXPORT RepaintUpdater {
@@ -1804,6 +1808,7 @@ KDE_NO_CDTOR_EXPORT ViewArea::ViewArea (QWidget *, View * view, bool paint_bg)
     setAcceptDrops (true);
     //new KAction (i18n ("Fullscreen"), KShortcut (Qt::Key_F), this, SLOT (accelActivated ()), m_collection, "view_fullscreen_toggle");
     setMouseTracking (true);
+    setFocusPolicy (Qt::ClickFocus);
     kapp->installX11EventFilter (this);
 }
 
@@ -1845,13 +1850,7 @@ KDE_NO_EXPORT void ViewArea::fullScreen () {
     m_fullscreen = !m_fullscreen;
     m_view->controlPanel()->fullscreenAction->setChecked (m_fullscreen);
 
-#ifdef KMPLAYER_WITH_CAIRO
-    if (surface->surface) {
-        cairo_surface_destroy (surface->surface);
-        surface->surface = 0L;
-        d->destroyBackingStore ();
-    }
-#endif
+    d->clearSurface (surface.ptr ());
     emit fullScreenChanged ();
 }
 
@@ -1876,9 +1875,21 @@ KDE_NO_EXPORT void ViewArea::accelActivated () {
     m_view->controlPanel()->fullscreenAction->trigger ();
 }
 
+KDE_NO_EXPORT void ViewArea::keyPressEvent (QKeyEvent *e) {
+    if (surface->node) {
+        QString txt = e->text ();
+        if (!txt.isEmpty ())
+            surface->node->document ()->message (MsgAccessKey,
+                    (void *)(long) txt[0].unicode ());
+    }
+}
+
 KDE_NO_EXPORT void ViewArea::mousePressEvent (QMouseEvent * e) {
     if (surface->node) {
-        MouseVisitor visitor (this, MsgEventClicked, e->x(), e->y());
+        MouseVisitor visitor (this, MsgEventClicked,
+                Matrix (surface->bounds.x (), surface->bounds.y (),
+                    surface->xscale, surface->yscale),
+                e->x(), e->y());
         surface->node->accept (&visitor);
     }
 }
@@ -1891,7 +1902,10 @@ KDE_NO_EXPORT void ViewArea::mouseMoveEvent (QMouseEvent * e) {
     if (e->state () == Qt::NoButton)
         m_view->mouseMoved (e->x (), e->y ());
     if (surface->node) {
-        MouseVisitor visitor (this, MsgEventPointerMoved, e->x(), e->y());
+        MouseVisitor visitor (this, MsgEventPointerMoved,
+                Matrix (surface->bounds.x (), surface->bounds.y (),
+                    surface->xscale, surface->yscale),
+                e->x(), e->y());
         surface->node->accept (&visitor);
         setCursor (visitor.cursor);
     }
@@ -1919,7 +1933,8 @@ KDE_NO_EXPORT void ViewArea::syncVisual () {
             surface->surface = d->createSurface (width (), height ());
             swap_rect = IRect (ex, ey, ew, eh);
             CairoPaintVisitor visitor (surface->surface,
-                    Matrix (surface->bounds.x(), surface->bounds.y(), 1.0, 1.0),
+                    Matrix (surface->bounds.x(), surface->bounds.y(),
+                        surface->xscale, surface->yscale),
                     swap_rect,
                     palette ().color (backgroundRole ()), true);
             surface->node->accept (&visitor);
@@ -1927,10 +1942,11 @@ KDE_NO_EXPORT void ViewArea::syncVisual () {
         } else if (!rect.isEmpty ()) {
             merge = cairo_surface_create_similar (surface->surface,
                     CAIRO_CONTENT_COLOR, ew, eh);
-            Matrix m (surface->bounds.x(), surface->bounds.y(), 1.0, 1.0);
-            m.translate (-ex, -ey);
             {
-                CairoPaintVisitor visitor (merge, m, IRect (0, 0, ew, eh),
+                CairoPaintVisitor visitor (merge,
+                        Matrix (surface->bounds.x()-ex, surface->bounds.y()-ey,
+                            surface->xscale, surface->yscale),
+                        IRect (0, 0, ew, eh),
                         palette ().color (backgroundRole ()), true);
                 surface->node->accept (&visitor);
             }
@@ -2005,38 +2021,20 @@ KDE_NO_EXPORT void ViewArea::updateSurfaceBounds () {
                 ? h
                 : (Single) m_view->controlPanel()->maximumSize ().height ())
         : Single (0);
-    Mrl *mrl = surface->node ? surface->node->mrl () : NULL;
+
     int scale = m_view->controlPanel ()->scale_slider->sliderPosition ();
-    int nw = w * scale / 100;
-    int nh = h * scale / 100;
-    x += (w - nw) / 2;
-    y += (h - nh) / 2;
-    w = nw;
-    h = nh;
-    if (m_view->keepSizeRatio () &&
-            w > 0 && h > 0 && mrl && !mrl->size.isEmpty ()) {
-        double wasp = (double) w / h;
-        double masp = (double) mrl->size.width / mrl->size.height;
-        if (wasp > masp) {
-            Single tmp = w;
-            w = masp * h;
-            x += (tmp - w) / 2;
-        } else {
-            Single tmp = h;
-            h = Single (w / masp);
-            y += (tmp - h) / 2;
-        }
-        surface->xscale = 1.0 * w / mrl->size.width;
-        surface->yscale = 1.0 * h / mrl->size.height;
-    } else {
-        surface->xscale = 1.0;
-        surface->yscale = 1.0;
+    if (scale != 100) {
+        int nw = w * 1.0 * scale / 100;
+        int nh = h * 1.0 * scale / 100;
+        x += (w - nw) / 2;
+        y += (h - nh) / 2;
+        w = nw;
+        h = nh;
     }
     if (surface->node) {
-        surface->bounds = SRect (x, y, w, h);
+        d->resizeSurface (surface.ptr ());
+        surface->resize (SRect (x, y, w, h));
         surface->node->message (MsgSurfaceBoundsUpdate, (void *) true);
-    } else {
-        surface->resize (SRect (x, y, w, h), true);
     }
     scheduleRepaint (IRect (0, 0, width (), height ()));
 }
@@ -2054,10 +2052,7 @@ KDE_NO_EXPORT void ViewArea::resizeEvent (QResizeEvent *) {
     bool auto_hide = m_view->controlPanelMode () == View::CP_AutoHide;
     h -= Single (auto_hide ? 0 : hcp) - hsb;
     // now scale the regions and check if video region is already sized
-    if (surface->node)
-        d->destroyBackingStore ();
     updateSurfaceBounds ();
-    d->resizeSurface (surface.ptr ());
 
     // finally resize controlpanel and video widget
     if (m_view->controlPanel ()->isVisible ())
@@ -2091,11 +2086,7 @@ KDE_NO_EXPORT Surface *ViewArea::getSurface (Mrl *mrl) {
 #ifdef KMPLAYER_WITH_CAIRO
         setAttribute (Qt::WA_OpaquePaintEvent, false);
         setAttribute (Qt::WA_PaintOnScreen, false);
-        if (surface->surface) {
-            cairo_surface_destroy (surface->surface);
-            surface->surface = 0L;
-            d->destroyBackingStore ();
-        }
+        d->clearSurface (surface.ptr ());
 #endif
     }
     scheduleRepaint (IRect (0, 0, width (), height ()));
